package egovframework.rte.cmmn.helper;

import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;

import javax.xml.transform.TransformerConfigurationException;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

import egovframework.rte.cmmn.util.Constants;
import egovframework.rte.cmmn.util.LogUtil;
import egovframework.rte.cmmn.util.StringUtil;

public class CmmnTransVOMap {

	public boolean isList;
	
	private String name;
	private LinkedHashMap<String, Object> mapVO;
	private boolean isEscaping;
	
	public CmmnTransVOMap() {
		init(false, "", false);
	}

	public CmmnTransVOMap(boolean isEscaping) {
		init(isEscaping, name, isList);
	}

	public CmmnTransVOMap(String name, boolean isList) {
		init(false, name, isList);
	}
	
	public CmmnTransVOMap(String name, boolean isList, boolean isEscaping) {
		init(isEscaping, name, isList);
	}
	
	private void init(boolean isEscaping, String name, boolean isList) {
		this.name = name;
		this.isList = isList;
		this.isEscaping = isEscaping;
		mapVO = new LinkedHashMap<String, Object>();
	}
	
	public void add(String key, Object value) throws Exception {
		if (key == null || key.length() < 0)
			throw new Exception("key name is empty");
		if (value == null)
			throw new Exception("value is null");
		
		mapVO.put(key, value);
	}
	
	public Object get(String key) throws Exception {
		
		return mapVO.get(key);
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}

	public int getObjectCount() {
		return mapVO.size();
	}
	
	public int getElementFromMap(Document dom, Element elParent) throws Exception {
		int nReturn = Constants.SUCCESS;
		try {
            String key = "";
            String value = "";
            for(Iterator eachKeyIterator = mapVO.keySet().iterator(); eachKeyIterator.hasNext(); )
            {
                key = (String)eachKeyIterator.next();
                Object object = get(key);
                if (object instanceof CmmnTransVOMap) {
                	// map include map list
                	CmmnTransVOMap map = (CmmnTransVOMap)object;
                	if (map.isList) {
                		Element elList = (Element)dom.createElement(map.getName());
                		
                		if (map.getElementFromListMap(dom, elList, map.getName()) != Constants.SUCCESS) {
                			throw new Exception("fail to extract list element!");
                		}
                		// list element add
                		elParent.appendChild(elList);
                	}
                	continue;
                }
                
                String tempVal = (String)object;
    			Element elKey = (Element)dom.createElement(key);
    			if (isEscaping) {
    				value = StringUtil.escapingTagString(value);
    			} else {
    				value = tempVal;
    			}
    			Text txtVal = (Text)dom.createTextNode(value);
    			elKey.appendChild(txtVal);
                
    			elParent.appendChild(elKey);
            }            
			
		}catch(TransformerConfigurationException tce){
			nReturn = Constants.XML_PARSER_ERR;
			LogUtil.trace(CmmnTransformHelper.ISDEBUG, tce.getMessage());
		} catch (Exception e) {
			nReturn = Constants.XML_PARSER_ERR;
			LogUtil.trace(CmmnTransformHelper.ISDEBUG, e.getMessage());
		}
		if (nReturn != Constants.SUCCESS) 
			throw new Exception("failed to create xml element.");
		
		return nReturn;
	}
	
	public int getElementFromListMap(Document dom, Element elParent, String parentName) throws Exception {
		int nReturn = Constants.SUCCESS;
		try {
			String key = "";
			for(Iterator eachKeyIterator = mapVO.keySet().iterator(); eachKeyIterator.hasNext(); )
			{
				key = (String)eachKeyIterator.next();
				Object object = get(key);

				if (object instanceof CmmnTransVOMap) {
					// map include map list
					CmmnTransVOMap mapItem = (CmmnTransVOMap)object;
					Element elItem = (Element)dom.createElement(parentName+"Item");
					
					if (mapItem.getElementFromMap(dom, elItem) != Constants.SUCCESS) {
						throw new Exception("failed to create xml element on list.");
					}
					elParent.appendChild(elItem);
				}
			}            
			
		}catch(TransformerConfigurationException tce){
			nReturn = Constants.XML_PARSER_ERR;
			LogUtil.trace(CmmnTransformHelper.ISDEBUG, tce.getMessage());
		} catch (Exception e) {
			nReturn = Constants.XML_PARSER_ERR;
			LogUtil.trace(CmmnTransformHelper.ISDEBUG, e.getMessage());
		}
		if (nReturn != Constants.SUCCESS)
			throw new Exception("failed to create xml element.");
		
		return nReturn;
	}
	
}
