package egovframework.rte.cmmn.helper;

import java.io.StringWriter;
import java.lang.reflect.Field;
import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.springframework.dao.DataAccessException;
import org.springframework.transaction.TransactionException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

import egovframework.rte.cmmn.util.Constants;
import egovframework.rte.cmmn.util.LogUtil;
import egovframework.rte.cmmn.util.StringUtil;
import egovframework.rte.fdl.cmmn.exception.EgovBizException;
import egovframework.rte.ptl.mvc.tags.ui.pagination.PaginationInfo;


/**
 *  Class Name : CmmnTransformHelper.java
 *  Description : transform form vo(or dto) object to xml 
 *  Modification Information
 * 
 *     수정일         수정자                   수정내용
 *   -------    --------    ---------------------------
 *
 *  @author 
 *  @since 2010. 9. 1.
 *  @version 1.0
 *  @see 
 * 
 */

public class CmmnTransformHelper {

	/** debug trace 출력 여부 */
	public final static boolean ISDEBUG = false;
	
	/** xml root tag */
	public final static String ROOTTAG  = "tcpor";
	/** default xml start tag of vo or list */
	public final static String OBJECTTAG = "item";
	public final static String LISTTAG   = "list";
	
	/** output xml encoding */
	private String encoding;
	/** html or script tag escaping 여부 */
	private boolean isEscaping;
	
	/** 특정 메서드에서 호출되었을경우에만 element에 attribute를 추가하기위한 변수*/
	private boolean theMethodCaller = false;
	/** 유형별 이미지 호출할 아이폰,안드로이드 공통 주소*/
	private String typeImgUrl;
	
	/** 디자인 최종버전(폴더명으로) 공통 주소*/
	private String chImgVarInfo;
	
	/** 동영상 또는 사진용량 제한 크기*/
	private String mediaSize = "2";
	
	/** 이미지 최대 첨부 갯수 */
	private String ckimgSzCt;
	
	/** 이미지 사이즈 */
	private String ckadImgSz;
	
	/** 사이트 주소 */
	private String cksiteAdd;
	
	/** 사이트 주소 */
	private String newAdress;
	
	DocumentBuilderFactory domBuilderFactory = null;
	DocumentBuilder domBuilder = null;
	Document dom = null;

	/**
	 * vo(dto)의 member variable 또는 vo(dto) member variable에 포함된 list 각 요소의 member variable에서
	 * 출력할 멤버변수를 걸러내는 filter list
	 * key값이 "", null 인 경우가 vo(dto) 자체 필터.
	 * key값은 vo에 포함된 member variable
	 */
	private FilterMap filterList;
	private ParamsMap paramsMap;

	/** 
	 * constructor
	 */
	public CmmnTransformHelper() {
		init("UTF-8", false);
	}
	
	/** 
	 * constructor
	 */
	public CmmnTransformHelper(boolean isEscaping) {
		init("UTF-8", isEscaping);
	}
	
	/**
	 * initialization object
	 * @param encoding - xml encoding
	 * @param isEscaping - escaping html tag or not 
	 */
	private void init(String encoding, boolean isEscaping) {
		boolean isException = false;
		try {
			this.encoding = encoding;
			this.isEscaping = isEscaping;
			domBuilderFactory = DocumentBuilderFactory.newInstance();
			domBuilder = domBuilderFactory.newDocumentBuilder();
			dom = domBuilder.newDocument();
			filterList = new FilterMap();
			paramsMap = null;
			
		} catch (ParserConfigurationException e) {
			LogUtil.trace(true, e.getMessage());
			isException = true;
		}catch(Exception e){  
			LogUtil.trace(true, e.getMessage());  
			isException = true;
		}  
		finally {
			if (isException) {
				domBuilderFactory = null;
				domBuilder = null;
				dom = null;
				filterList = null;
				paramsMap = null;
			}
		}
	}
	
	/**
	 * vo 필드 중 보여줄 것만 필터링하는 필터링 리스트
	 * @param key - vo member variable name (단, vo root filter는 "" 또는 null)
	 * @param filter - 최종 조회하고자 하는 member variable list 
	 */
	public void addFilter(String key, List<String> filter){
		if (filter == null) {
			return;
		}
		filterList.addFilter(key, filter);
	}
	
	public void addParam(String key, String value) {
		if (paramsMap == null) {
			paramsMap = new ParamsMap();
		}
		paramsMap.addParam(key, value);
	}
	
	public String getEncoding() {
		return encoding;
	}

	public void setEncoding(String encoding) {
		this.encoding = encoding;
	}
	
	public boolean isEscaping() {
		return isEscaping;
	}

	public void setEscaping(boolean isEscaping) {
		this.isEscaping = isEscaping;
	}
	
	public String transMwListXmlT(Object vo, String startTag){
		String xml = "";
		String itemTag = OBJECTTAG;
		if (startTag != null && startTag.length() > 0) {
			itemTag = startTag;
		}
		
		try {
			if (vo == null)
				return getErrorXML(Constants.NO_DATA_ERR);
			
			if (dom == null)
				return getErrorXML(Constants.XML_PARSER_ERR);
			
			
			Element root = (Element)dom.createElement(ROOTTAG);
			dom.appendChild(root);

			Element eRoot = dom.getDocumentElement();
			if (setReusltCodeXML(eRoot) > 0) {
				return getErrorXML(Constants.XML_PARSER_ERR); 
			}
			
			// ##### transformer from VO to Map
			CmmnTransVOMap mapData = getTransMapFromObject(vo);
			CmmnTransVOMap targetMap = null;
			// root filter key is "" or null or FilterMap.rootFilter
			if (filterList.getFilter("") != null) {
				targetMap = getFilterResultMap("", mapData);
			} else {
				targetMap = mapData;
			}

			Element elItem = (Element)dom.createElement(itemTag);
			
			
			targetMap.getElementFromMap(dom, elItem);
			
			eRoot.appendChild(elItem);
			
			// etc params elements add
			if (paramsMap != null && paramsMap.getParamsCount() > 0) {
				paramsMap.setParamsElement(dom, eRoot, isEscaping);
			}
			
			
			StringWriter sw = new StringWriter();
			Transformer transformer = TransformerFactory.newInstance().newTransformer();  
			//출력 포멧 설정  
			transformer.setOutputProperty(OutputKeys.ENCODING, encoding);  
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");  
			
			transformer.transform(new DOMSource(dom), new StreamResult(sw));
			
			xml = sw.getBuffer().toString();
			
		}catch(TransformerConfigurationException tce){
			LogUtil.trace(true, tce.getMessage());
			xml = getErrorXML(Constants.XML_PARSER_ERR);
		}catch(Exception e){  
			LogUtil.trace(true, e.getMessage());
			xml = getErrorXML(Constants.XML_PARSER_ERR);
		}
		
		return xml;
	}
	
	
	
	/**
	 * transformer from vo(dto) to xml 
	 * @param vo - vo or dto
	 * @param startTag - xml parent tag (root tag 아님)
	 * @return xml string
	 */
	public String transObjectToXml(Object vo, String startTag) {
		String xml = "";
		String itemTag = OBJECTTAG;
		if (startTag != null && startTag.length() > 0) {
			itemTag = startTag;
		}
		
		try {
			if (vo == null)
				return getErrorXML(Constants.NO_DATA_ERR);
			
			if (dom == null)
				return getErrorXML(Constants.XML_PARSER_ERR);
			
			
			Element root = (Element)dom.createElement(ROOTTAG);
			dom.appendChild(root);

			Element eRoot = dom.getDocumentElement();
			if (setReusltCodeXML(eRoot) > 0) {
				return getErrorXML(Constants.XML_PARSER_ERR); 
			}
			
			// ##### transformer from VO to Map
			CmmnTransVOMap mapData = getTransMapFromObject(vo);
			CmmnTransVOMap targetMap = null;
			// root filter key is "" or null or FilterMap.rootFilter
			if (filterList.getFilter("") != null) {
				targetMap = getFilterResultMap("", mapData);
			} else {
				targetMap = mapData;
			}

			Element elItem = (Element)dom.createElement(itemTag);
			
			targetMap.getElementFromMap(dom, elItem);
			
			eRoot.appendChild(elItem);
			
			// etc params elements add
			if (paramsMap != null && paramsMap.getParamsCount() > 0) {
				paramsMap.setParamsElement(dom, eRoot, isEscaping);
			}
			
			StringWriter sw = new StringWriter();
			Transformer transformer = TransformerFactory.newInstance().newTransformer();  
			//출력 포멧 설정  
			transformer.setOutputProperty(OutputKeys.ENCODING, encoding);  
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");  
			
			transformer.transform(new DOMSource(dom), new StreamResult(sw));
			
			xml = sw.getBuffer().toString();
			
		}catch(TransformerConfigurationException tce){
			LogUtil.trace(true, tce.getMessage());
			xml = getErrorXML(Constants.XML_PARSER_ERR);
		}catch(Exception e){  
			LogUtil.trace(true, e.getMessage());
			xml = getErrorXML(Constants.XML_PARSER_ERR);
		}
		
		return xml;
	}

	/**
	 * transformer from List<Map<String,Object>> object to xml 
	 * @param list - multi row data List<Map<String,Object>> list
	 * @param startTag - xml parent tag (root tag 아님)
	 * @return xml string
	 */
	public String transMapToXml(List<Map<String,Object>> list, String startTag) {
		return transListMapToXml(list, null, startTag);
	}

	/**
	 * transformer from List<Map<String,Object>> object to xml 
	 * @param list - multi row data List<Map<String,Object>> list
	 * @param paginationInfo - page information 
	 * @param startTag - xml parent tag (root tag 아님)
	 * @return xml string
	 */
	public String transMapToXml(List<Map<String,Object>> list, PaginationInfo paginationInfo, String startTag) {
		return transListMapToXml(list, paginationInfo, startTag);
	}
	
	/**
	 * transformer from List object to xml 
	 * @param list - multi row data list
	 * @param startTag - xml parent tag (root tag 아님)
	 * @return xml string
	 */
	public String transListToXml(List list, String startTag) {
		
		return transListMapToXml(list, null, startTag);
	}
	
	/**
	 * transformer from List object to xml 
	 * @param list - multi row data list
	 * @param startTag - xml parent tag (root tag 아님)
	 * @return xml string
	 */
	public String transListToXml(List list, String startTag, String url, String size, String chImgInfo, String ckimgSzCt, String cksiteAdd, String newAdress, String ckadImgSz) {
		
		StackTraceElement[] s  = Thread.currentThread().getStackTrace();
		
		for(int i = 0 ; i < s.length ; i ++ ) {
			StackTraceElement stack = s[i];
			String tmpMethodInStack = stack.getMethodName();
			if("selectListLurTypeOpenVer".equalsIgnoreCase(tmpMethodInStack)){
				this.theMethodCaller = true;
				this.typeImgUrl      = url;
				this.mediaSize 	     = size;
				this.chImgVarInfo    = chImgInfo;
				this.ckimgSzCt       = ckimgSzCt;
				this.cksiteAdd       = cksiteAdd;
				this.newAdress       = newAdress;
				this.ckadImgSz       = ckadImgSz;
				break;
			}
		}
		
		return transListMapToXml(list, null, startTag);
	}

	/**
	 * transformer from List object to xml 
	 * @param list - multi row data list
	 * @param paginationInfo - page information 
	 * @param startTag - xml parent tag (root tag 아님)
	 * @return xml string
	 */
	public String transListToXml(List list, PaginationInfo paginationInfo, String startTag) {
		
		return transListMapToXml(list, paginationInfo, startTag);
	}

	/**
	 * transformer from List object to xml 
	 * @param list - multi row data list
	 * @param paginationInfo - page information 
	 * @param startTag - xml parent tag (root tag 아님)
	 * @param isMapList - list가 List<Map<String,Object>> 인지 여부  -- 삭제 2011.10.05
	 * @return xml string
	 */
	private String transListMapToXml(List list, PaginationInfo paginationInfo, String startTag) {
		
		String xml = "";
		String itemsTag = LISTTAG;
		if (startTag != null && startTag.length() > 0) {
			itemsTag = startTag;
		}
		try {
			if (list == null || list.isEmpty())
				return getErrorXML(Constants.NO_DATA_ERR);
			
			if (dom == null)
				return getErrorXML(Constants.XML_PARSER_ERR);
			
			
			Element root = (Element)dom.createElement(ROOTTAG);
			dom.appendChild(root);

			Element eRoot = dom.getDocumentElement();
			if (setReusltCodeXML(eRoot) > 0) {
				return getErrorXML(Constants.XML_PARSER_ERR); 
			}
			Element elItems = (Element)dom.createElement(itemsTag);	
			if (paginationInfo != null) {
				elItems.setAttribute("totalrecordcount", String.valueOf(paginationInfo.getTotalRecordCount()));
				elItems.setAttribute("currentpageno", String.valueOf(paginationInfo.getCurrentPageNo()));
				elItems.setAttribute("totalpagecount", String.valueOf(paginationInfo.getTotalPageCount()));
			}
			
			if(theMethodCaller){
				elItems.setAttribute("s1a", "0");
				elItems.setAttribute("s2b", "0");
				elItems.setAttribute("s3c", this.ckadImgSz);
				elItems.setAttribute("s4imgurl",    this.typeImgUrl);
				elItems.setAttribute("s5videosize", this.mediaSize);
				elItems.setAttribute("s6imgvar",    this.chImgVarInfo);
				elItems.setAttribute("s7imgadd",    this.ckimgSzCt);
				elItems.setAttribute("s8siteadd",   this.cksiteAdd);
				elItems.setAttribute("s9newadress", this.newAdress);
			}
			
			// ##### transformer - from VO to Map
			CmmnTransVOMap mapData = getTransMapFromList(list, itemsTag);
			
			mapData.getElementFromListMap(dom, elItems, itemsTag);
			
			eRoot.appendChild(elItems);
			
			// etc params elements add
			if (paramsMap != null && paramsMap.getParamsCount() > 0) {
				paramsMap.setParamsElement(dom, eRoot, isEscaping);
			}
			
			StringWriter sw = new StringWriter();
			Transformer transformer = TransformerFactory.newInstance().newTransformer();  
			// set output format  
			transformer.setOutputProperty(OutputKeys.ENCODING, encoding);  
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");  
			
			transformer.transform(new DOMSource(dom), new StreamResult(sw));
			
			xml = sw.getBuffer().toString();
			
		}catch(TransformerConfigurationException tce){
			LogUtil.trace(true, tce.getMessage());
			xml = getErrorXML(Constants.XML_PARSER_ERR);
		}catch(Exception e){  
			LogUtil.trace(true, e.getMessage());
			xml = getErrorXML(Constants.XML_PARSER_ERR);
		}
		
		return xml;
	}
	
	/**
	 * extract from multiple row data Map to multiple data CmmnTransVOMap object
	 * @param vo - multiple row data vo or dto object
	 * @return multiple data CmmnTransVOMap object
	 */
	public CmmnTransVOMap getTransMapFromMap(Object vo) throws Exception {
		CmmnTransVOMap mapVO = new CmmnTransVOMap(isEscaping);
		
        try {
        	
    		if (vo != null && vo instanceof Map) {
				Map<String, Object> mapEach = (Map<String, Object>)vo;
				
	            String key = "";
	            String value = "";
	            for(Iterator eachKeyIterator = mapEach.keySet().iterator(); eachKeyIterator.hasNext(); )
	            {
	                key = (String)eachKeyIterator.next();	                
					value = "" + (mapEach.get(key)==null ? "" : mapEach.get(key));                			
	    			LogUtil.trace(ISDEBUG, ">>>> each Map }}}}}}}} key : " + key + " / value : " + value);
	    			mapVO.add(key, value);
	            }            
    		}        	
        	
		} catch (Exception e) {
			mapVO = null;
			throw new Exception(e);
		}
		
		return mapVO;
	}
	
	/**
	 * extract from vo(dto) to map object
	 * @param vo - vo or dto
	 * @return CmmnTransVOMap object
	 */
	public CmmnTransVOMap getTransMapFromObject(Object vo) throws Exception {

		CmmnTransVOMap mapVO = new CmmnTransVOMap(isEscaping);
		
        Field[] fs = vo.getClass().getDeclaredFields();
        Method[] methods = vo.getClass().getDeclaredMethods();
        String strErrMsg ="";
        
        try {
        	
	        for (int i=0; i<fs.length; i++) {
	        	String fieldName = fs[i].getName();
	        	String fieldType = fs[i].getType().getName();
	        	
	        	LogUtil.trace(ISDEBUG, "------> fieldName :" + fieldName + " / type :" + fieldType);
	        	
	        	String val = "";
	        	strErrMsg = "";
        	
                for (int j=0; j<methods.length; j++) {
                	Method method = methods[j];

                	if (method.getName().equalsIgnoreCase("get"+fieldName)) {
                		
                		Object[] params = new Object[] {}; 
                		if (fieldType.equalsIgnoreCase("java.util.List")) {
                			List subList = (List)method.invoke(vo, params);
                			
                			if (subList != null) {
                				CmmnTransVOMap mapList = getTransMapFromList(subList, fieldName, fieldName);                				
                				mapVO.add(fieldName, mapList);
                			}
                			continue;
                		}
               			val = "" +  (method.invoke(vo, params)==null ? "" : method.invoke(vo, params));
                		
                		LogUtil.trace(ISDEBUG, "========------> field value :" + val); 
                		mapVO.add(fieldName, val);
                		break;
                	} 
                }
	        }
	        
		} catch (IllegalAccessException e)   {
			strErrMsg = " Method can not be accessed." + e.getMessage() ;
		} catch (IllegalArgumentException e) {
			strErrMsg = " Transfer factor of the method was wrong." + e.getMessage();
		} catch (InvocationTargetException e) {
			strErrMsg = " Method failed to execute.";
		} catch (NullPointerException e) {
			strErrMsg = e.getMessage();
		} catch (Exception e) {
			strErrMsg = e.getMessage();
		}
		
		if (strErrMsg.length() > 0)
			throw new Exception(strErrMsg);
		
        return mapVO;
	}
	
	/**
	 * extract from list to multiple data CmmnTransVOMap object
	 * -> list group root  
	 * @param list - mutiple row data list
	 * @param listName - list group tag name
	 * @return multiple row data CmmnTransVOMap object 
	 */
	public CmmnTransVOMap getTransMapFromList(List list, String listName) throws Exception {
		return getTransMapFromList(list, listName, null);
	}
	
	/**
	 * extract from list to multiple data CmmnTransVOMap object
	 * @param list - mutiple row data list
	 * @param listName - list group tag name
	 * @param filterKey - list 각각의 row item에서 보여줄 column filter name(or member variable name) 
	 * @return multiple row data CmmnTransVOMap object 
	 */
	public CmmnTransVOMap getTransMapFromList(List list, String listName, String filterKey) throws Exception {
		
		CmmnTransVOMap mapList = new CmmnTransVOMap(listName, true, isEscaping);
		
		for (int i=0; i<list.size(); i++) {
			LogUtil.trace(ISDEBUG, "<<" + i + ">> list item");
			Object vo = list.get(i);
			CmmnTransVOMap item = null;
			
    		if (vo != null && vo instanceof Map) {
    			item = getTransMapFromMap(vo);
    		} else {
    			item = getTransMapFromObject(vo);
    		}
			CmmnTransVOMap destItem = null;
			
			if (filterList.getFilter(filterKey) != null) {
				destItem = getFilterResultMap(filterKey, item);
			} else {
				destItem = item;
			}
			mapList.add("item"+i, destItem);
		}
		
		return mapList;
	}

	/**
	 * extract showing variable using filter in vo member variables
	 * @param filterKey - filter name
	 * @param srcMap - source vo member variables map
	 * @return extracted CmmnTransVOMap object
	 */
	public CmmnTransVOMap getFilterResultMap(String filterKey, CmmnTransVOMap srcMap) throws Exception {

		List<String> filter = filterList.getFilter(filterKey);
		if (filter == null) {
			// if there is no filter, return source map
			return srcMap;
		}
		
		CmmnTransVOMap map = new CmmnTransVOMap(isEscaping);
		
		try {
			for (int i=0; i<filter.size(); i++) {
				String colName = filter.get(i);
				
				Object findObj = srcMap.get(colName);
				if (findObj != null) {					
					map.add(colName, findObj);					
				}
			}
			
		} catch (Exception e) {
			throw new Exception(e.getMessage());
		}
		
		return map;
	}
	
	/**
	 * create result xml tag
	 * @return xml string
	 */
	public String getResultCodeXML() {
		String xml = "";
		
		try {
			if (dom == null)
				return getErrorXML(Constants.XML_PARSER_ERR);
			
			Element root = (Element)dom.createElement(ROOTTAG);
			dom.appendChild(root);

			Element eRoot = dom.getDocumentElement();
			if (setReusltCodeXML(dom, eRoot, "", 0) > 0) {
				return getErrorXML(Constants.XML_PARSER_ERR); 
			}
			
			StringWriter sw = new StringWriter();
			Transformer transformer = TransformerFactory.newInstance().newTransformer();  
			// set output format  
			transformer.setOutputProperty(OutputKeys.ENCODING, encoding);  
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");  
			
			transformer.transform(new DOMSource(dom), new StreamResult(sw));
			
			xml = sw.getBuffer().toString();
			
		}catch(TransformerConfigurationException tce){
			LogUtil.trace(true, tce.getMessage());
			xml = getErrorXML(Constants.XML_PARSER_ERR);
		}catch(Exception e){  
			LogUtil.trace(true, e.getMessage());
			xml = getErrorXML(Constants.XML_PARSER_ERR);
		}
		
		return xml;
	}

	/**
	 * set result xml data
	 * @param eRoot - xml root element
	 * @return result code (Constants code) 
	 */
	public int setReusltCodeXML(Element eRoot) {
		return setReusltCodeXML(dom, eRoot, "", 0);
	}
	
	/**
	 * set result xml data
	 * @param dom   - Document instance
	 * @param eRoot - xml root element
	 * @param errDesc  - error description
	 * @return result code (Constants code) 
	 */
	public static int setReusltCodeXML(Document dom, Element eRoot, String errDesc, int errCode) {
		int nRet = Constants.SUCCESS;
		int rsltCode = errCode==0 ? Constants.SUCCESS : errCode; 
		
		try {
			if (dom ==null)
				return Constants.XML_PARSER_ERR;
			
			Element eResult = (Element)dom.createElement("result");
			
			Element elCd = (Element)dom.createElement("result_code");
			Text eCdTxt = (Text)dom.createTextNode(Integer.toString(rsltCode));
			elCd.appendChild(eCdTxt);
			
			Element elDesc = (Element)dom.createElement("result_desc");
			Text eDescTxt = (Text)dom.createTextNode(errDesc);
			elDesc.appendChild(eDescTxt);
			
			eResult.appendChild(elCd);
			eResult.appendChild(elDesc);
			
			eRoot.appendChild(eResult);
			
		}catch(Exception e){
			LogUtil.trace(true, e.getMessage());
			nRet = Constants.XML_PARSER_ERR;
		}  
		
		return nRet;
	}

	/**
	 * create to return error xml tag (applied default encoding)
	 * @param nErr - error code
	 * @param e    - Exception object
	 * @return xml string
	 */
	public static String getErrorXML(int nErr, Exception e) {
		return getErrorXML(nErr, e, "UTF-8");
	}	

/*	*//**
	 * create to return error xml tag (applied default encoding)
	 * @param nErr - error code
	 * @param e    - Exception object
	 * @param encoding - setting encoding 
	 * @param defaultErr - err;
	 * @return xml string
	 *//*
	public static String getErrorXML(int nErr, Exception e, String encoding, String defaultErr) {
	}	
*/	
	/**
	 * create to return error xml tag (applied default encoding)
	 * @param nErr - error code
	 * @param e    - Exception object
	 * @param encoding - setting encoding 
	 * @return xml string
	 */
	public static String getErrorXML(int nErr, Exception e, String encoding) {
		String errDesc = "";
		int errCd = nErr;
		
		if (e instanceof DataAccessException) {
			LogUtil.trace(ISDEBUG, "--->>> DataAccessException Exception :" + e.getMessage());
			errCd = Constants.DATABASE_ACCESS_ERR;
		} else if (e instanceof TransactionException) {
			LogUtil.trace(ISDEBUG, "--->>> TransactionException Exception : " + e.getMessage());
			errCd = Constants.DATABASE_TRANSACTION_ERR;
		} else if (e instanceof EgovBizException) {
			LogUtil.trace(ISDEBUG, "--->>> EgovBizException Exception : " + e.getMessage());
			errCd = Constants.BIZ_PROCESS_ERR;
			errDesc = e.getMessage();
		} else {
			LogUtil.trace(ISDEBUG, "--->>> unknown Exception : " + e.getMessage());
			errDesc = e.getMessage();
		}
		
		return getErrorXML(errCd, errDesc, encoding);
	}

	/**
	 * create to return error xml tag (applied default encoding)
	 * @param nErr - error code
	 * @return xml string
	 */
	public static String getErrorXML(int nErr) {
		return getErrorXML(nErr, "", "UTF-8");
	}
	
	/**
	 * create to return error xml tag (applied default encoding)
	 * @param nErr - error code
	 * @param errDesc - error description
	 * @return xml string
	 */
	public static String getErrorXML(int nErr, String errDesc) {
		return getErrorXML(nErr, errDesc, "UTF-8");
	}
	
	/**
	 * create to return error xml tag (set ecoding)
	 * @param nErr - error code
	 * @param errDesc - error description
	 * @param encoding - setting encoding
	 * @return xml string
	 */
	public static String getErrorXML(int nErr, String errDesc, String encoding) {
		
		String xml = "";
		String err = errDesc;
		if (nErr == 0)
			return "";
		
		DocumentBuilderFactory domBF = null;
		DocumentBuilder builder = null;
		Document document = null;

		try {
			domBF = DocumentBuilderFactory.newInstance();
			builder = domBF.newDocumentBuilder();
			document = builder.newDocument();
			
			Element root = (Element)document.createElement(ROOTTAG);
			document.appendChild(root);
			Element eRoot = document.getDocumentElement();
			
			err = getErrorDesc(nErr, errDesc);
			
			if (setReusltCodeXML(document, eRoot, err, nErr) > 0) {
				return getDummyErrXml(nErr, err, encoding); 
			}
			StringWriter sw = new StringWriter();
			Transformer transformer = TransformerFactory.newInstance().newTransformer();  
			//출력 포멧 설정  
			transformer.setOutputProperty(OutputKeys.ENCODING, encoding);
			transformer.setOutputProperty(OutputKeys.INDENT, "yes");  
			
			transformer.transform(new DOMSource(document), new StreamResult(sw));
			
			xml = sw.getBuffer().toString();
			LogUtil.trace(ISDEBUG, xml);
			
		}catch(TransformerConfigurationException tce){
			LogUtil.trace(true, tce.getMessage());
			xml = getDummyErrXml(Constants.XML_PARSER_ERR, err, encoding);
		} catch (ParserConfigurationException pe) {
			LogUtil.trace(true, pe.getMessage());
			xml = getDummyErrXml(Constants.XML_PARSER_ERR, err, encoding);
		}catch(Exception e){  
			LogUtil.trace(true, e.getMessage());  
			xml = getDummyErrXml(Constants.UNKNOWN_ERR, err, encoding);
		}
		
		return xml;
	}
	
	public static String getDummyErrXml(int nErr, String errDesc, String encoding) {
		String xml = "<?xml version=\"1.0\" encoding=\"" + encoding + "\" ?>";
		xml += "<" + ROOTTAG + ">";
		xml += "<result>";
		xml += "<result_code>" + nErr + "</result_code>";
		xml += "<result_desc>";
		xml += getErrorDesc(nErr, errDesc);
		xml += "</result_desc>";
		xml += "</result>";
		xml += "</" + ROOTTAG + ">";
		
		return xml;
	}
	
	public static String getErrorDesc(int nErr, String err) {
		String errDesc = err;
		switch (nErr) {
		case Constants.UNKNOWN_ERR: 
			if (errDesc.length() == 0)
				errDesc = "알 수 없는 문제가 발생하였습니다.";
			break;
		case Constants.NO_DATA_ERR:
			if (errDesc.length() == 0)
				errDesc = "데이터가 없습니다.";
			break;
		case Constants.INVALID_PARAM_ERR:
			if (errDesc.length() == 0)
				errDesc = "잘못된 전달인자입니다.";
			break;
		case Constants.XML_PARSER_ERR:
			if (errDesc.length() == 0)
				errDesc = "XML parser error!";
			break;
		case Constants.DATABASE_ACCESS_ERR:
			if (errDesc.length() == 0)
				errDesc = "데이터베이스 처리에 문제가 발생하였습니다.";
			break;
		case Constants.DATABASE_TRANSACTION_ERR:
			if (errDesc.length() == 0)
				errDesc = "데이터 등록/수정/삭제에 문제가 발생하였습니다.";
			break;
		case Constants.BIZ_PROCESS_ERR:
			if (errDesc.length() == 0)
				errDesc = "업무 처리에 문제가 발생하였습니다.";
			break;
		case Constants.ALREADY_EXISTED_ERR:
			if (errDesc.length() == 0)
				errDesc = "이미 존재하는 데이터 입니다.";
			break;
		default:
			if (errDesc.length() == 0)
				errDesc = "xml paser error";
			break;
		}
		return errDesc;
	}
	
	class FilterMap {
		/** 
		 * filterMap key is member variable		  
		 */
		public final static String ROOTFILTER = "_F_Root_";
		private final LinkedHashMap<String, List<String>> mapFilter;
		
		/**
		 * Constructor
		 */
		public FilterMap() {
			mapFilter = new LinkedHashMap<String, List<String>>();
		}
		
		public void addFilter(String key, List<String> filter) {
			if (key == null || key.length() == 0) {
				key = ROOTFILTER;
			}
			mapFilter.put(key, filter);
		}
		
		public List<String> getFilter(String key) {
			if (key == null || key.length() == 0) {
				key = ROOTFILTER;
			}
			return mapFilter.get(key);
		}		
		
		public int getFilterCount() {
			return mapFilter.size();
		}
	}
	
	class ParamsMap {
		/*
		 * set parameters vo(or dto) 외의 parameter 
		 */
		private HashMap<String, String> mapParams;
		
		/** Constructor */
		public ParamsMap() {
			//mapParams = new HashMap<String, String>();
			mapParams = null;
		}
		
		public int getParamsCount() {
			if (mapParams == null)
				return 0;
			return mapParams.size();
		}
		
		public void addParam(String key, String value) {
			if (mapParams == null) {
				mapParams = new HashMap<String, String>();
			}
			mapParams.put(key, value);
		}
		
		public String getParam(String key) {
			if (mapParams == null)
				return null;
			return mapParams.get(key);
		}
		
		/**
		 * if etc parameters is existed, add xml paramters element  
		 */
		public int setParamsElement(Document dom, Element elRoot, boolean isEscaping) throws Exception {
			int nReturn = Constants.SUCCESS;
			if (getParamsCount() == 0)
				return nReturn;

			if (dom ==null)
				return Constants.XML_PARSER_ERR;

			int nCount = 0;
			try {
				
				Element elParams = (Element)dom.createElement("params");
				
				for(Iterator eachKeyIterator = mapParams.keySet().iterator(); eachKeyIterator.hasNext(); )
				{
					String key = (String)eachKeyIterator.next();
					String tempVal = (String)mapParams.get(key);
					String value = "";

					if (value != null) {
		    			Element elKey = (Element)dom.createElement(key);
		    			if (isEscaping) {
		    				value = StringUtil.escapingTagString(value);
		    			} else {
		    				value = tempVal;
		    			}
		    			Text txtVal = (Text)dom.createTextNode(value);
		    			elKey.appendChild(txtVal);
		    			
		    			elParams.appendChild(elKey);
		    			nCount++;
					}
				}
				
				if (nCount > 0) {
					elRoot.appendChild(elParams);
				}
					
			} catch (Exception e) {
				nReturn = Constants.XML_PARSER_ERR;
				LogUtil.trace(true, e.getMessage());
			}
			if (nReturn != Constants.SUCCESS)
				throw new Exception("fail create parameter element!");
			
			return nReturn;
		}
	}
}
