package egovframework.rte.cmmn.util;

import java.io.UnsupportedEncodingException;
import java.net.URLEncoder;
import java.util.HashMap;

import org.springframework.stereotype.Component;


/************************************************************************************************
* 1.isEmpty : 공백 검사
* 2. nullToEmpty : null값 공백치환
* 3. nullToEmptyChk : checkbox null값 "N"으로 치환                            
*************************************************************************************************/
@Component("StringUtil")
public class StringUtil{

//  value   hex     dec 
//  "       0x22    34
//  '       0x27    39
//  space   0x20    32
//  enter   0x0D    13  --> only UNIX
//  enter   0x0A    10  --> UNIX & MS
	private static Integer doubleQuot = new Integer(34);
	private static Integer singleQuot = new Integer(39);
	private static Integer space      = new Integer(32);
	private static Integer enterUnix  = new Integer(13);
	private static Integer enter      = new Integer(10);
	
/**
 * null 또는 "" 공백 검사.
 * @param String
 * @return boolean
 */
    public static boolean isEmpty(String str){
        if (str == null || str.length() < 1) return true;

        return false;
    }  
/**
 * null 값을 "" 공백 문자로 바꿔준다.
 * @param String
 * @return String
 */
    public static String nullToEmpty(String str) {
    	String strVal = str;
        if(strVal == null || strVal.length() < 1) {
            return "";
        }
        strVal = replaceHtmlTag(strVal);
        return strVal;
    }	

    /**
     * 값이 null이거나 값이 없을 경우 "" 지정된 Default 문자로 바꿔준다.
     * @param String
     * @return String
     */
	public static String nullToStrDefVal(String str, String strDefault) {
		String strVal = str;
	    if(strVal == null || strVal.length() < 1) {
	        return strDefault;
	    }
        strVal = replaceHtmlTag(strVal);
	    return strVal;
	}	
    
	/**
	 * null 값을 0 문자로 바꿔준다.
	 * @param String
	 * @return String
	 */
    public static long nullToZero(String str) {
        if(str == null || str.length() < 1) {
            return 0;
        }
        String strVal = replaceHtmlTag(str);
        return Long.valueOf(strVal);
    }	

    /**
     * 값이 null이거나 값이 없을 경우 "" 지정된 Default 숫자로 바꿔준다.
     * @param String
     * @return long
     */
	public static long nullToNumDefVal(String str, long lDefault) {
	    if(str == null || str.length() < 1) {
	        return lDefault;
	    }
	    String strVal = replaceHtmlTag(str);
	    return Long.valueOf(strVal);
	}	
    
	/**
	 * null 값을 0 문자로 바꿔준다.
	 * @param String
	 * @return String
	 */
    public static int nullToZeroInt(String str) {
        if(str == null || str.length() < 1) {
            return 0;
        }
        String strVal = replaceHtmlTag(str);
        return Integer.valueOf(strVal);
    }	

    /**
     * null 값을 "" 공백 문자로 바꿔준다.
     * @param String
     * @return String
     */
    public static String nullToEmptyOfTag(String str) {
    	String strVal = str;
        if(strVal == null || strVal.length() < 1) {
            return "";
        }
        return strVal;
    }	

        /**
         * 값이 null이거나 값이 없을 경우 "" 지정된 Default 문자로 바꿔준다.
         * @param String
         * @return String
         */
    
	/**
	 * null 값을 오늘 일자로 바꿔준다. (형식: yyyy-MM-dd)
	 * @param String
	 * @return String
	 */
    public static String nullToDate(String str) {
        if(str == null || str.length() < 1) {
            return DateUtil.getDate();
        }
        return str;
    }	
    
	/**
	 * null 값을 끝 일자로 바꿔준다. (2999-12-31)
	 * @param String
	 * @return String
	 */
    public static String nullToEndDate(String str) {
        if(str == null || str.length() < 1) {
            return "2999-12-31";
        }
        return str;
    }	

	/**
	 * null 값을 "N" 문자로 바꿔준다.
	 * @param String
	 * @return String
	 */
    public static String nullToEmptyChk(String str) {
        if(str == null  || str.length() < 1 || !str.equals(Constants.YES)) {
            return Constants.NO;
        }
        return str;
    }
    /**
     * 파일경로에서 확장자만 추출해서 반환한다.
     * @param String
     * @return String
     */
    public static String getFileExt(String filePath) {
        int index = filePath.lastIndexOf(".");

        if (index == -1) {
            return "";
        }

        return filePath.substring(index + 1, filePath.length());
    }

    /**
     * 파일경로에서 파일명과 확장자를 추출해서 반환한다.
     * @param String
     * @return String
     */
    public static HashMap<String, String> getFileNameAndExt(String filePath) {
        HashMap<String, String> mapFileInfos = new HashMap<String, String>();
    	String fpath = StringUtil.replaceEx(filePath, "\\", "/");
    	int index = fpath.lastIndexOf(".");
        int fPos = (fpath.lastIndexOf("/") != -1 ? fpath.lastIndexOf("/") : 0);

        if (index == -1) {
            return null;
        }
        mapFileInfos.put("name", fpath.substring(fPos, index));
        mapFileInfos.put("ext", fpath.substring(index + 1, filePath.length()));

        return mapFileInfos;
    }
    
    /**
     * "-"를 제거해준다.
     * @param String
     * @return String
     */
    
    public static String removeDash(String s){
        if ( s == null )
        {
            return null;
        }

        if ( s.indexOf("-") != -1 )
        {
            StringBuffer buf = new StringBuffer();

            for(int i=0;i<s.length();i++)
            {
                char c = s.charAt(i);
                if ( c != '-')
                {
                    buf.append(c);
                } 
            }
            return buf.toString();
        } 
        return s;
    }
    
    /**
     * 개행문자 <BR>붙이기
     * @param String
     * @return String
     */
    public static String appendHtmlBr(String comment)
    {
       int length = comment.length();
    
       StringBuffer buffer = new StringBuffer();
         if(comment.indexOf("<table") == -1){    
           for (int i = 0; i < length; ++i){
              String comp = comment.substring(i, i+1);
              
              if ("\r".compareTo(comp) == 0){
                 comp = comment.substring(++i, i+1);
                 if ("\n".compareTo(comp) == 0){
                    buffer.append("<BR>\r");
                 }
                 else{
                    buffer.append("\r");
                 }
              }
    
              buffer.append(comp);
           }
         }
		 else{
			 return comment;
		 }

       return buffer.toString();
    }
    
    /**
     * 암호화 인코딩
     * @param String
     * @return String
     */
    public static String encoding(String pwd) throws Exception{     
        int i = 0;
            int emp_no1 = 0;
            int emp_no2 = 0;
            int emp_no3 = 0;
            int result1 = 0;
            byte ch ;       
            String result = "";
              
        try{ 
            int ccc = pwd.length();
            for (i=0;i<3;i++ ){           
                    ch  = (byte)pwd.charAt(i);        
                
                    if(i==0){
                        switch(ch){
                            case 0: 
                            case 1:
                            case 2:
                            case 3:
                            case 4:
                            case 5:
                            case 6:
                            case 7:
                            case 8:
                            case 9:
                                emp_no1 = (int)ch;
                                break ;
                            default : 
                                emp_no1 = 4;
                                break;
                        }
                    }
                    else if(i==1){
                        switch(ch){
                            case 0: 
                            case 1:
                            case 2:
                            case 3:
                            case 4:
                            case 5:
                            case 6:
                            case 7:
                            case 8:
                            case 9:
                                emp_no2 = 5;
                                break;
                            default : 
                                emp_no2 = 3;
                                break;
                        }
                    }
                    else if(i==2){
                        switch(ch){
                            case 0: 
                            case 1:
                            case 2:
                            case 3:
                            case 4:
                            case 5:
                            case 6:
                            case 7:
                            case 8:
                            case 9:
                                emp_no1 = (int)ch;
                               break;
                            default : 
                                emp_no1 = 2;
                               break;
                        }
                    }           
            }       

            for(i = 0; i<ccc; i++){
                  int aaa = 0;
                  aaa = i%6;
                  char ddd = pwd.charAt(i); 

                  switch(aaa){
                     case 0:
                        result1 = (int)ddd - emp_no2;
                        result = result + (char)result1;
                        break;
                     case 1:
                        result1 = (int)ddd - emp_no3;
                        result = result + (char)result1;
                        break;
                            
                    case 2:
                        result1 = (int)ddd - emp_no1;
                        result = result + (char)result1;
                        break;      
                        
                    case 3:
                        result1 = (int)ddd - emp_no3 - emp_no2;
                        result = result + (char)result1;
                        break;      
                    case 4:
                        result1 = (int)ddd - emp_no1 - emp_no3;
                        result = result + (char)result1;
                        break;                   
                    case 5:
                        result1 = (int)ddd - emp_no1 - emp_no2;
                        result = result + (char)result1;
                        break;
                    default:
                    	break;
                 }                  
            }       
            
        }       
        catch (Exception e){
            LogUtil.trace(true, "** Error in encoding : " + e.getMessage());
        }
        return result;
    } 
    
    /**
     * 암호화 디코딩
     * @param String
     * @return String
     */
    public static String decoding(String pwd) throws Exception{     
        int i = 0;
            int emp_no1 = 0;
            int emp_no2 = 0;
            int emp_no3 = 0;
            int result1 = 0;
            byte ch ;       
            String result = "";
              
        try{ 
            int ccc = pwd.length();
            for (i=0;i<3;i++ ){           
                    ch  = (byte)pwd.charAt(i);        
                
                    if(i==0){
                        switch(ch)
                        {
                            case 0: 
                            case 1:
                            case 2:
                            case 3:
                            case 4:
                            case 5:
                            case 6:
                            case 7:
                            case 8:
                            case 9:
                                emp_no1 = (int)ch;
                                break ;
                            default : 
                                emp_no1 = 4;
                                break;
                        }
                    }
                    else if(i==1){
                        switch(ch){
                            case 0: 
                            case 1:
                            case 2:
                            case 3:
                            case 4:
                            case 5:
                            case 6:
                            case 7:
                            case 8:
                            case 9:
                                emp_no2 = 5;
                                break;
                            default : 
                                emp_no2 = 3;
                                break;
                        }
                    }
                    else if(i==2){
                        switch(ch){
                            case 0: 
                            case 1:
                            case 2:
                            case 3:
                            case 4:
                            case 5:
                            case 6:
                            case 7:
                            case 8:
                            case 9:
                                emp_no1 = (int)ch;
                               break;
                            default : 
                                emp_no1 = 2;
                               break;
                        }
                    }           
            }       

            for(i = 0; i<ccc; i++){
                  int aaa = 0;
                  aaa = i%6;
                  char ddd = pwd.charAt(i); 

                  switch(aaa){
                     case 0:
                        result1 = (int)ddd + emp_no2;
                        result = result + (char)result1;
                        break;
                     case 1:
                        result1 = (int)ddd + emp_no3;
                        result = result + (char)result1;
                        break;
                            
                    case 2:
                        result1 = (int)ddd + emp_no1;
                        result = result + (char)result1;
                        break;      
                        
                    case 3:
                        result1 = (int)ddd + emp_no3 + emp_no2;
                        result = result + (char)result1;
                        break;      
                    case 4:
                        result1 = (int)ddd + emp_no1 + emp_no3;
                        result = result + (char)result1;
                        break;                   
                    case 5:
                        result1 = (int)ddd + emp_no1 + emp_no2;
                        result = result + (char)result1;
                        break;  
                    default:
                    	break;
                 }
            }       
            
        }       
        catch (Exception e){
        	LogUtil.trace(true, "** Error in encoding : " + e.getMessage());
        }
        return result;
    }
    
    /**
     * html 테그를 html 화면 출력용으로 변환한다.
     * @param str   문자열
     * @return   문자열
     */
    public static String htmlToString(String str) {
    	
    	String res = str;
    	res = escapingTagString(res);
		//res = res.replace("\n", "<br>\n");
		res = res.replace("  ", " &nbsp;");
	
	    return res;
    }

    /**
     * html 테그를 xml 화면 출력용으로 변환한다.
     * @param str   문자열
     * @return   문자열
     */
    public static String escapingTagString(String str) {
    	
    	String res = str;
		res = res.replace("'", "&#39;");
		res = res.replace("&", "&amp;");
		res = str.replace("<", "&lt;");
		res = res.replace(">", "&gt;");
		res = res.replace("\"", "&quot;");
		res = res.replace("\'", "&apos;");
		res = res.replace("\t", " &nbsp; &nbsp;");
	
	    return res;
    }
    
    /**
     * html 을 문자열로 한것을 html 테그로 변경한다 .
     * @param str
     * @return
     */
     public static String convertForHTML(String str) {
		
    	 String res = null;
		 res = str.replace((char)doubleQuot.byteValue(), (char)singleQuot.byteValue());
		 res = res.replace((char)enterUnix.byteValue(), (char)space.byteValue());
		 res = res.replace((char)enter.byteValue(), (char)space.byteValue());
		   
		 return res;
	}
    
	/**
	 * 문자열에 포함되어 있는 < 와 > 를 없애 버린다
	 * @param contents
	 * @return
	 */
     public static String removeTags(String contents) {

    	 StringBuffer source = new StringBuffer(contents);
	     StringBuffer target = new StringBuffer();
	     boolean deleteChar = false;
	   
	     for (int i=0; i < source.length(); i++) {
	    	 if (source.charAt(i) == '<') {
	    		 deleteChar = true;
	    	 } else if (source.charAt(i) == '>') {
	    		 deleteChar = false;
	    	 } else if (!deleteChar) {
	    		 target.append(source.charAt(i));
	    	 }
	     }
	   
	     return target.toString();
     }
       
	/**
	* IMG 태그의 src 필드의 내용에 URLEncoder를 적용시킨다.<br/>
	* IMG 태그의 사용은 반드시 대소문자를 구별하여 IMG, src, 큰 따옴표(")를 사용하여야 한다. <br/>
	* (예: <IMG src="예제.jpg"> )<br/>
	* SpaceEditor를 사용하는 경우 위와 같은 형식으로 세팅된다.<br/>
	*
	* @param contents 변경할 내용
	* @param enc encoding type (예: EUC-KR)
	* @return 변경된 스트링
	* @throws UnsupportedEncodingException
	*/
     public static String convertImgURLEncode(String contents, String enc) throws UnsupportedEncodingException {
    	 String returnContents = contents;
    	 if (contents == null || enc == null)
    		 return null;
  
    	 int endIdx = 0;
    	 for (int i=0; i < contents.length(); i=endIdx+1) {
    		 int imgIdx = contents.indexOf("<IMG", i);
    		 if (imgIdx == -1) break;
   
    		 int srcIdx = contents.indexOf("src", imgIdx);
    		 if (srcIdx == -1) break;
   
    		 int beginIdx = contents.indexOf("\"", srcIdx);
    		 if (beginIdx == -1) break;
   
    		 endIdx = contents.indexOf("\"", beginIdx+1);
    		 if (endIdx == -1) break;
   
    		 String replaceStr = contents.substring(beginIdx + 1, endIdx);
    		 returnContents = contents.replace(replaceStr, URLEncoder.encode(replaceStr, enc));
    	 }
      
    	 return returnContents;
     }    
     
 	/**
 	 * 대상 String 에서 특정 String을 찾아서 다른 String으로 대체하여 return
 	 * @param str
 	 *            대상 String
 	 * @param from
 	 *            찾는 String
 	 * @param to
 	 *            취환할 String
 	 */
 	public static String replaceEx( String str, String from, String to ) {
 		String sResult = "";
 		try {
 			if( str == null || str.length() == 0 || from == null || from.length() == 0
 					|| to == null || to.length() == 0 ) return str;

 			StringBuffer sb = null;

 			sb = new StringBuffer( str.length() * 2 );
 			String posString = str.toLowerCase();
 			String cmpString = from.toLowerCase();
 			int i = 0;
 			boolean done = false;
 			while( i < str.length() && !done ) {
 				int start = posString.indexOf( cmpString, i );
 				if( start == -1 ) {
 					done = true;
 				} else {
 					sb.append( str.substring( i, start ));
 					sb.append(to);
 					i = start + from.length();
 				}
 			}
 			if( i < str.length() ) {
 				sb.append( str.substring( i ) );
 			}

 			sResult = sb.toString();
 		} catch( Exception e ) {
 			sResult = str;
 		}

 		return sResult;
 	}
     
 	/**
 	 * 대상 String 에서 < 와 > & 를 &lt; 와 &gt; 와 &amp;로 바꿔서 return
 	 * @param sHTML
 	 *            대상 String
 	 */
 	public static String replaceHtmlTag( String sHTML )  {
 		if( sHTML == null || sHTML.trim().equals( "" ) ) return "";

 		String sResult = "";
 		StringBuffer sbResult = null;

 		try {
 			String s = "";
 			sbResult = new StringBuffer();

 			for( int i = 0; i < sHTML.length(); i++ ) {
 				s = sHTML.substring( i, i + 1 );

 				if( "<".equals( s ) ) {
 					sbResult.append( "&lt;" );
 				} else if( "<".equals( s ) ) {
 					sbResult.append( "&gt;" );
 				} else if( "\"".equals( s ) ) {
 					sbResult.append( "&quot;" );
 				} else if( "'".equals( s ) ) {
 					sbResult.append( "&#39;" );
 				} else if( "&".equals( s ) ) {
 					sbResult.append( "&amp;" );
 				} else {
 					sbResult.append( s );
 				}
 			}

 			sResult = sbResult.toString();
 		} finally {
 			sbResult = null;
 		}
 		sResult = replaceEx( sResult, "/*", "" );
 		sResult = replaceEx( sResult, "*/", "" );
 		sResult = replaceEx(sResult,"%","");
 		sResult = replaceEx(sResult,"%00",""); 
 		return sResult;
 	}

	//XSS 처리
	public static String cleanXSS(String str)
    {
		String strVal = str;
		strVal = strVal.replaceAll("<", "&lt;").replaceAll(">", "&gt;");
		strVal = strVal.replaceAll("\\(", "&#40;").replaceAll("\\)", "&#41;");
		strVal = strVal.replaceAll("'", "&#39;");
		strVal = strVal.replaceAll("eval\\((.*)\\)", "");
		strVal = strVal.replaceAll("[\\\"\\'][\\s]*javascript:(.*)[\\\"\\']", "\"\"");
		strVal = strVal.replaceAll("[\\\"\\'][\\s]*JAVASCRIPT:(.*)[\\\"\\']", "\"\"");
		strVal = strVal.replaceAll("&lt;/script&gt;", "");
		strVal = strVal.replaceAll("&lt;SCRIPT&gt;", "");
		strVal = strVal.replaceAll("&lt;/script&gt;", "");
		strVal = strVal.replaceAll("&lt;SCRIPT&gt;", "");
        return strVal;
    }
 	
 	/**
 	 * 대상 String 에서 < 와 > & 를 &lt; 와 &gt; 와 &amp;로 바꿔서 return
 	 * @param sHTML
 	 *            대상 String
 	 */
 	public static boolean checkScriptTag( String data ) throws Exception {

 		boolean bExistTag = false;
 		if (data == null || data.length() <= 0) {
 			return false;
 		}

 		try {
 			String sHTML = data; //new String(data);
 			sHTML = sHTML.toLowerCase().replace(" ", "");
 			sHTML = sHTML.replace("\n", "");
 			
 			if (sHTML.length() <= 0)
 				return false;
 			
 			if (sHTML.indexOf("<script") != -1 || 
 				sHTML.indexOf("<iframe") != -1 ||
 				sHTML.indexOf("style")   != -1 ||
 				sHTML.indexOf("src")     != -1) {
 				bExistTag = true;
 			} 

 		} catch (Exception e) {
 			throw new Exception(e);
 		}
 		
 		return bExistTag;
 	}
 	
    /**
     * 문자열이 지정한 길이를 초과했을때 지정한길이에다가 해당 문자열을 붙여주는 메서드.
     * @param source 원본 문자열 배열
     * @param output 더할문자열
     * @param slength 지정길이
     * @return 지정길이로 잘라서 더할분자열 합친 문자열
     */
    public static String cutString(String source, String output, int slength) {
        String returnVal = null;
        if (source != null) {
            if (source.length() > slength) {
                returnVal = source.substring(0, slength) + output;
            } else
                returnVal = source;
        }
        return returnVal;
    }

    /**
     * 문자열이 지정한 길이를 초과했을때 해당 문자열을 삭제하는 메서드
     * @param source 원본 문자열 배열
     * @param slength 지정길이
     * @return 지정길이로 잘라서 더할분자열 합친 문자열
     */
    public static String cutString(String source, int slength) {
        String result = null;
        if (source != null) {
            if (source.length() > slength) {
                result = source.substring(0, slength);
            } else
                result = source;
        }
        return result;
    }
 	
    
    /**
     * <pre>
     * 널값일 경우 대체 객체를 반환한다.
     * </pre>
     *
     * @param source sourceObj
     * @param alernative alernativeObj
     * @return Object
     */
    public static Object nvl(Object source, Object alernative) {
        if (source == null|| source.toString().equals("")) {
            return alernative;
        }
        return source;
    }
	
    
}