package egovframework.rte.dsdsm.cmmcode.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.springframework.stereotype.Service;

import egovframework.rte.dsdsm.cmmcode.service.CodeService;
import egovframework.rte.fdl.cmmn.EgovAbstractServiceImpl;
import egovframework.rte.psl.dataaccess.util.EgovMap;

@Service("CodeService")
public class CodeServiceImpl extends EgovAbstractServiceImpl implements CodeService{
	
    @Resource(name="CodeServiceDAO")
    private CodeServiceDAO CodeServiceDAO;

    /**
	 * 공통코드(대분류) 총 갯수
	 * @exception Exception
	 */
    public int getCodeGenListCount(HashMap h) throws Exception {
		return CodeServiceDAO.getCodeGenListCount(h);
	}
    
    /**
	 * 공통코드(대분류) 목록
	 * @exception Exception
	 */
    public List getCodeGenList(HashMap h) throws Exception {
        return CodeServiceDAO.getCodeGenList(h);
    }
	
    /**
   	 * 공통코드(대분류) 상세정보
   	 * @exception Exception
   	 */
    public List getCodeGenView(HashMap h) throws Exception {
       return CodeServiceDAO.getCodeGenView(h);
    }
    
    /**
	 * 조회된 공통코드(대분류)의 세부코드 총 갯수
	 * @exception Exception
	 */
    public int getCodeGenDetailListCount(HashMap h) throws Exception {
		return CodeServiceDAO.getCodeGenDetailListCount(h);
	}
    
    /**
	 * 조회된 공통코드(대분류)의 세부코드 목록
	 * @exception Exception
	 */
    public List getCodeGenDetailList(HashMap h) throws Exception {
        return CodeServiceDAO.getCodeGenDetailList(h);
    }
    
    /**
	 * 공통코드(소분류) 총 갯수
	 * @exception Exception
	 */
    public int getCodeListCount(HashMap h) throws Exception {
		return CodeServiceDAO.getCodeListCount(h);
	}
    
    /**
	 * 공통코드(소분류) 목록
	 * @exception Exception
	 */
    public List getCodeList(HashMap h) throws Exception {
        return CodeServiceDAO.getCodeList(h);
    }
    
    /**
   	 * 공통코드(소분류) 상세정보
   	 * @exception Exception
   	 */
    public List getCodeView(HashMap h) throws Exception {
       return CodeServiceDAO.getCodeView(h);
    }
    
    /**
   	 * 공통코드(대분류) ID 중복체크
   	 * @exception Exception
   	 */
    @Override
	public Map<String, Object> checkCodeId(Map<String, String> map) throws Exception {
		
		Map <String,Object> result = CodeServiceDAO.checkCodeId(map);
		
		@SuppressWarnings("unchecked")
		Map<String, Object> one = (Map<String, Object>)result.get("ONE");
		
		if(one==null){
			result.put("SUCCESS", true);
			result.put("MESSAGE", "공통코드ID를 사용할 수 있습니다.");
		}else{
			result.put("SUCCESS", false);
			result.put("MESSAGE", "공통코드ID가 중복되었습니다.<br>다른 ID를 입력해주세요.");
		}
		
		return result;
	}
    
    /**
   	 * 공통코드(소분류) ID 중복체크
   	 * @exception Exception
   	 */
    @Override
	public Map<String, Object> checkCode(Map<String, String> map) throws Exception {
		
		Map <String,Object> result = CodeServiceDAO.checkCode(map);
		
		@SuppressWarnings("unchecked")
		Map<String, Object> one = (Map<String, Object>)result.get("ONE");
		
		if(one==null){
			result.put("SUCCESS", true);
			result.put("MESSAGE", "공통코드(소분류)ID를 사용할 수 있습니다.");
		}else{
			result.put("SUCCESS", false);
			result.put("MESSAGE", "공통코드(소분류)ID가 중복되었습니다.<br>다른 ID를 입력해주세요.");
		}
		
		return result;
	}
    
    /**
   	 * 공통코드(대분류) 등록
   	 * @exception Exception
   	 */
    public Map<String, Object> codeGenReg(HashMap h) throws Exception {
		Map <String,Object> result = CodeServiceDAO.codeGenReg(h);
		
		String cnt = (String)result.get("CNT");
		
		if(cnt==null){
			result.put("SUCCESS", true);
		}else{
			result.put("SUCCESS", false);
		}
		
		return result;
	}
    
    /**
   	 * 공통코드(대분류) 수정
   	 * @exception Exception
   	 */
    public Map<String, Object> codeGenEdit(HashMap h) throws Exception {
		Map <String,Object> result = CodeServiceDAO.codeGenEdit(h); //대분류ID 수정
		
		Integer cnt = (Integer)result.get("CNT");
		
		if(cnt.intValue()>0){
			result.put("SUCCESS", true);
			result.put("MESSAGE", "공통코드(대분류) 수정이 완료되었습니다");
		} else {
			result.put("SUCCESS", false);
			result.put("MESSAGE", "공통코드(대분류) 수정에 실패하였습니다.\r\n잠시 후 다시 시도바랍니다.");
		}
		
		return result;
	}
    
    /**
   	 * 공통코드(대분류) 삭제
   	 * @exception Exception
   	 */
    public Map<String, Object> codeGenDel(HashMap h) throws Exception {
		Map <String,Object> result = CodeServiceDAO.codeGenDel(h); //대분류코드 삭제x
		Integer cnt = (Integer)result.get("CNT");
		
		Map <String,Object> result2 = CodeServiceDAO.codeGenDtDel(h); //소분류코드 삭제
		
		if(cnt.intValue()>0){
			result.put("SUCCESS", true);
			result.put("MESSAGE", "공통코드(대분류) 삭제가 완료되었습니다");
		} else {
			result.put("SUCCESS", false);
			result.put("MESSAGE", "공통코드(대분류) 삭제에 실패하였습니다.\r\n잠시 후 다시 시도바랍니다.");
		}
		
		return result;
	}
    
    /**
   	 * 공통코드(소분류) 등록
   	 * @exception Exception
   	 */
    public Map<String, Object> codeReg(HashMap h) throws Exception {
		Map <String,Object> result = CodeServiceDAO.codeReg(h);
		
		String cnt = (String)result.get("CNT");
		
		if(cnt==null){
			result.put("SUCCESS", true);
		}else{
			result.put("SUCCESS", false);
		}
		
		return result;
	}
    
    /**
   	 * 공통코드(소분류) 수정
   	 * @exception Exception
   	 */
    public Map<String, Object> codeEdit(HashMap h) throws Exception {
		Map <String,Object> result = CodeServiceDAO.codeEdit(h);
		
		Integer cnt = (Integer)result.get("CNT");
		
		if(cnt.intValue()>0){
			result.put("SUCCESS", true);
			result.put("MESSAGE", "공통코드(소분류) 수정이 완료되었습니다");
		} else {
			result.put("SUCCESS", false);
			result.put("MESSAGE", "공통코드(소분류) 수정에 실패하였습니다.\r\n잠시 후 다시 시도바랍니다.");
		}
		
		return result;
	}
    
    /**
   	 * 공통코드(대분류) 삭제
   	 * @exception Exception
   	 */
    public Map<String, Object> codeDel(HashMap h) throws Exception {
		Map <String,Object> result = CodeServiceDAO.codeDel(h); //대분류코드 삭제x
		Integer cnt = (Integer)result.get("CNT");
		
		if(cnt.intValue()>0){
			result.put("SUCCESS", true);
			result.put("MESSAGE", "공통코드(소분류) 삭제가 완료되었습니다");
		} else {
			result.put("SUCCESS", false);
			result.put("MESSAGE", "공통코드(소분류) 삭제에 실패하였습니다.\r\n잠시 후 다시 시도바랍니다.");
		}
		
		return result;
	}
    
    /**
	 * 조회된 업무코드 목록
	 * @exception Exception
	 */
    public List getJobCdList() throws Exception {
        return CodeServiceDAO.getJobCdList();
    }    
}
