package egovframework.rte.dsdsm.messenger.util;

import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.jivesoftware.smack.SmackException;
import org.jivesoftware.smack.XMPPException;
import org.jivesoftware.smack.chat.Chat;
import org.jivesoftware.smack.chat.ChatManager;
import org.jivesoftware.smack.tcp.XMPPTCPConnection;
import org.jivesoftware.smack.tcp.XMPPTCPConnectionConfiguration;
import org.jivesoftware.smackx.iqregister.AccountManager;
import org.jivesoftware.smackx.search.ReportedData;
import org.jivesoftware.smackx.search.UserSearch;
import org.jivesoftware.smackx.search.UserSearchManager;
import org.jivesoftware.smackx.xdata.Form;

import egovframework.rte.cmmn.util.EgovProperties;
import egovframework.rte.dsdsm.cmm.util.CryptoUtil;

public class MessengerUtil {
	
	
	/*
	 * oepnFire 특정 유저에게 메쇠지를 보냄
	 */
	public static  void sendSingleUserMsg(String id, String msg) throws Exception {

		String openFireIp          = EgovProperties.getProperty("Globals.openFireIp");
		String openFireAdminId     = EgovProperties.getProperty("Globals.openFireAdminId");
		String openFireAdminPasswd = EgovProperties.getProperty("Globals.openFireAdminPasswd");
		
		XMPPTCPConnectionConfiguration config = XMPPTCPConnectionConfiguration.builder()
				.setServiceName(openFireIp)
				.setHost(openFireIp)
				.setPort(5222).build();
		
		XMPPTCPConnection connection = new XMPPTCPConnection(config);
		
		try {
			connection.connect();
			connection.login(openFireAdminId, openFireAdminPasswd);
			
			ChatManager chatManager = ChatManager.getInstanceFor(connection);
			
			//ID에 반드시 @IP를 붙여야 메쇠지가 전송 됨..
			String test2JID = id+"@"+openFireIp;
			
			Chat chat = chatManager.createChat(test2JID);
			chat.sendMessage(msg);
			
		} catch (XMPPException e) {
			e.printStackTrace();
		} catch (SmackException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		}finally {
			connection.disconnect();
		}
	}
		
		
	/*
	 * oepnFire 전체 유저에게 메쇠지를 보냄
	 */
	public static  void sendAllUserMsg(String msg) throws Exception {
			
			String openFireIp = EgovProperties.getProperty("Globals.openFireIp");
			XMPPTCPConnectionConfiguration config = XMPPTCPConnectionConfiguration.builder()
					.setServiceName(openFireIp)
					.setHost(openFireIp)
					.setPort(5222).build();
			
			XMPPTCPConnection connection = new XMPPTCPConnection(config);
			
			String openFireAdminId     = EgovProperties.getProperty("Globals.openFireAdminId");
			String openFireAdminPasswd = EgovProperties.getProperty("Globals.openFireAdminPasswd");
			
			try {
					connection.connect();
					connection.login(openFireAdminId, openFireAdminPasswd);
				
				 	UserSearchManager manager = new UserSearchManager(connection);
				    
				    String searchFormString = "search." + connection.getServiceName();
				    //System.out.println("*** SearchForm: " + searchFormString);
				    
				    Form searchForm = manager.getSearchForm(searchFormString);
				    Form answerForm = searchForm.createAnswerForm();

				    UserSearch userSearch = new UserSearch();
				    answerForm.setAnswer("Username", true);
				    answerForm.setAnswer("search", "*");
				    
				    ReportedData results = userSearch.sendSearchForm(connection, answerForm, searchFormString);
				    if (results != null) {
				        List<ReportedData.Row> rows = results.getRows();
				        for (ReportedData.Row row : rows) {
				        	ChatManager chatManager = ChatManager.getInstanceFor(connection);
				        	Chat chat = chatManager.createChat((row.getValues("Username")).get(0) +"@"+openFireIp);
				        	System.out.println("spark user : "+row.getValues("Username"));
				    		chat.sendMessage(msg);
				        }
				    } else {
				        System.out.println( "No result found");
				    }
				
			} catch (XMPPException e) {
				e.printStackTrace();
			} catch (SmackException e) {
				e.printStackTrace();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}finally {
				connection.disconnect();
			}

	}  

	
	/*
	 * oepnFire 서버에 사용자를 등록함
	 * Group 까지 등록하면 좋으나, Group은 OpenFire 에서 변경해줘야 한다.
	 * Api가 아직.. 부실(?) ...  
	 */
	public static  void addUser(String id, String userName) throws Exception {
		String openFireIp = EgovProperties.getProperty("Globals.openFireIp");
		XMPPTCPConnectionConfiguration config = XMPPTCPConnectionConfiguration.builder()
				.setServiceName(openFireIp)
				.setHost(openFireIp)
				.setPort(5222).build();
		
		XMPPTCPConnection connection = new XMPPTCPConnection(config);
		
		String openFireAdminId     = EgovProperties.getProperty("Globals.openFireAdminId");
		String openFireAdminPasswd = EgovProperties.getProperty("Globals.openFireAdminPasswd");
		
		try {
				connection.connect();
				connection.login(openFireAdminId, openFireAdminPasswd);
		
				AccountManager accountManager = AccountManager.getInstance(connection);
				Map<String, String> attributes = new HashMap();
				attributes.put("name", userName);
       
	           if (accountManager.supportsAccountCreation()) {
	               accountManager.sensitiveOperationOverInsecureConnection(true);
	               accountManager.createAccount(id , "123456", attributes );  // 메신저 초기에는 123456 으로 셋팅.
	           }
		} catch (Exception e) {
			   e.printStackTrace();
		}finally {
			connection.disconnect();
       }
	}
 

	
	/*
	public static void main(String[] agu)	{
		
		String openFireIp = "192.168.0.99";
		XMPPTCPConnectionConfiguration config = XMPPTCPConnectionConfiguration.builder()
				.setServiceName(openFireIp)
				.setHost(openFireIp)
				.setPort(5222).build();
		
		XMPPTCPConnection connection = new XMPPTCPConnection(config);

		
		String openFireAdminId = "pmsadmin";
		String openFireAdminPasswd = "123456";
		
		try {
			
		connection.connect();
		connection.login(openFireAdminId, openFireAdminPasswd);
			
	    
	    UserSearchManager manager = new UserSearchManager(connection);
	    
	    String searchFormString = "search." + connection.getServiceName();
	    
	    System.out.println("*** SearchForm: " + searchFormString);
	    
	    Form searchForm = manager.getSearchForm(searchFormString);
	    Form answerForm = searchForm.createAnswerForm();

	    UserSearch userSearch = new UserSearch();
	    answerForm.setAnswer("Username", true);
	    answerForm.setAnswer("search", "*");
	    
	    


	    ReportedData results = userSearch.sendSearchForm(connection, answerForm, searchFormString);
	    if (results != null) {
	        List<ReportedData.Row> rows = results.getRows();
	        for (ReportedData.Row row : rows) {
	        	System.out.println("row: " + row.getValues("Username").get(0) +"@192.168.0.99");
	        	
	        	ChatManager chatManager = ChatManager.getInstanceFor(connection);
	    		
	        	
	        	Chat chat = chatManager.createChat(row.getValues("Username").get(0) +"@192.168.0.99");
	    	
	    		chat.sendMessage("Test All User");
	    		
	        }
	    } else {
	        System.out.println( "No result found");
	    }

	    } catch (SmackException.NoResponseException e) {
	        e.printStackTrace();
	    } catch (XMPPException.XMPPErrorException e) {
	        e.printStackTrace();
	    } catch (SmackException.NotConnectedException e) {
	        e.printStackTrace();
	    } catch (SmackException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} catch (XMPPException e) {
			e.printStackTrace();
		}finally {
			connection.disconnect();
		}
	}
	*/
	
	
	/*
	 * openFilre 부서 정보 가져옴
	 */
	
	/*
	public static void main(String[] agu)	{
		
		String msg ="test..";
		
		String openFireIp = "192.168.0.173";
		XMPPTCPConnectionConfiguration config = XMPPTCPConnectionConfiguration.builder()
				.setServiceName(openFireIp)
				.setHost(openFireIp)
				.setPort(5222).build();
		
		XMPPTCPConnection connection = new XMPPTCPConnection(config);
		
		String openFireAdminId     = "pmsadmin";//EgovProperties.getProperty("Globals.openFireAdminId");
		String openFireAdminPasswd = "123456";//EgovProperties.getProperty("Globals.openFireAdminPasswd");
		
		try {
				connection.connect();
				connection.login(openFireAdminId, openFireAdminPasswd);
			
			 	UserSearchManager manager = new UserSearchManager(connection);
			    
			    String searchFormString = "search." + connection.getServiceName();
			    //System.out.println("*** SearchForm: " + searchFormString);
			    
			    Form searchForm = manager.getSearchForm(searchFormString);
			    Form answerForm = searchForm.createAnswerForm();

			    UserSearch userSearch = new UserSearch();
			    answerForm.setAnswer("Username", true);
			    answerForm.setAnswer("search", "*");
			    
			    ReportedData results = userSearch.sendSearchForm(connection, answerForm, searchFormString);
			    if (results != null) {
			        List<ReportedData.Row> rows = results.getRows();
			        for (ReportedData.Row row : rows) {
			        	ChatManager chatManager = ChatManager.getInstanceFor(connection);
			        	Chat chat = chatManager.createChat( (row.getValues("Username")).get(0) +"@192.168.0.173" );
			        	System.out.println("spark user : "+(row.getValues("Username")).get(0) +"@"+openFireIp);
			        	System.out.println(msg);
			        	chat.sendMessage(msg);
			        }
			    } else {
			        System.out.println( "No result found");
			    }
			
		} catch (XMPPException e) {
			e.printStackTrace();
		} catch (SmackException e) {
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}finally {
			//connection.disconnect();
		}
	}
	*/
	



}