package egovframework.rte.dsdsm.projectSchedule.web;

import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.apache.commons.lang.StringEscapeUtils;
import org.codehaus.jackson.map.ObjectMapper;
import org.codehaus.jettison.json.JSONArray;
import org.codehaus.jettison.json.JSONObject;
import org.codehaus.jettison.json.JSONTokener;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;

import egovframework.rte.cmmn.util.StringUtil;
import egovframework.rte.dsdsm.cmm.service.CmmService;
import egovframework.rte.dsdsm.cmm.service.CommonDataService;
import egovframework.rte.dsdsm.projectSchedule.service.ProjScService;
import egovframework.rte.dsdsm.projectSchedule.service.vo.assigs;
import egovframework.rte.dsdsm.projectSchedule.service.vo.project;
import egovframework.rte.dsdsm.projectSchedule.service.vo.resources;
import egovframework.rte.dsdsm.projectSchedule.service.vo.tasks;

@Controller
public class ProjScController {
	
	@Resource(name = "ProjScService")
    private ProjScService ProjScService;

	@Resource(name = "CmmService")
    private CmmService CmmService;
	
	@Resource(name="CommonDataService")
	private CommonDataService commonDataService;
	
	/**
     * 프로젝트 리스트
	 * @throws Exception 
     */
    @RequestMapping(value="/project/psList.do")
    public String psList(HttpServletRequest request, Model model) throws Exception {
    	
    	HashMap<Object, Object> map = new HashMap<Object, Object>();
    	
    	map.put("roleId", "MANAGER000");
		model.addAttribute("authority", CmmService.getAuthority(map));
		
		map.put("roleId", "SYSMANAGER");
		model.addAttribute("authority2", CmmService.getAuthority(map));
		
    	return "/dsdsm/project/psList";
    }
    
    /**
     * 프로젝트 작업별 담당자 등록
	 * @throws Exception 
     */
    @RequestMapping(value="/project/taskChargerManage.do")
    public String taskChargerManage(@RequestParam HashMap<Object,Object> param,HttpServletRequest request, Model model) throws Exception {
    	
    	@SuppressWarnings("rawtypes")
		List chargerList = ProjScService.getChargerList(param);
		
		ObjectMapper mapper2 = new ObjectMapper();
   		String deptCodesJsonStr2 = mapper2.writeValueAsString(chargerList);
   		
   		model.addAttribute("NODE_LIST", deptCodesJsonStr2);
    	
   		@SuppressWarnings("rawtypes")
		List taskChargerList = ProjScService.getTaskChargerList(param);
   		
   		String deptCodesJsonStr = mapper2.writeValueAsString(taskChargerList);
   		
   		model.addAttribute("TASKCHARGER_LIST", deptCodesJsonStr);
   		
   		model.addAttribute("projectid",param.get("projectid"));
   		model.addAttribute("taskid",param.get("taskid"));
   		
    	return "/dsdsm/project/psTaskCharger";
    }
    
    /**
     * 프로젝트 담당자 관리
	 * @throws Exception 
     */
    @RequestMapping(value="/project/psCharger.do")
    public String psCharger(@RequestParam HashMap<Object,Object> param,HttpServletRequest request, Model model) throws Exception {
    	
    	String projectid = request.getParameter("projectid");
    	
    	model.addAttribute("projectid",projectid);
    	
    	@SuppressWarnings("rawtypes")
		List deptCodes = commonDataService.getDeptCode();
    	
    	ObjectMapper mapper = new ObjectMapper();
   		String deptCodesJsonStr = mapper.writeValueAsString(deptCodes);
    	
    	model.addAttribute("DEPT_CODE", deptCodesJsonStr);
    	
    	@SuppressWarnings("rawtypes")
		List chargerList = ProjScService.getChargerList(param);
		
		ObjectMapper mapper2 = new ObjectMapper();
   		String deptCodesJsonStr2 = mapper2.writeValueAsString(chargerList);
   		
   		model.addAttribute("NODE_LIST", deptCodesJsonStr2);
   		
    	return "/dsdsm/project/psCharger";
    }
    
	/**
     * 사용자 목록을 불러온다.
     * @throws Exception 
     */
	@RequestMapping(value = "/project/getUserList.json")
	public String getUserList(@RequestParam HashMap<Object,Object> param,HttpServletRequest request, ModelMap model, HttpSession session) throws Exception {

    	List userList = ProjScService.getUserList(param);
    	
    	model.put("userList",userList);
    	
		return "jsonView";
	}
	
    /**
     * 프로젝트 목록을 불러온다.
     * @throws Exception 
     */
	@RequestMapping(value = "/project/getProjectList.do")
	public String getProjectList(@RequestParam HashMap<Object,Object> param,HttpServletRequest request, ModelMap model, HttpSession session) throws Exception {

		HashMap<Object, Object> pageinMap = new HashMap<Object, Object>();

		float pageNo = Integer.parseInt(request.getParameter("pageNo").toString());
		float pageSize = Integer.parseInt(request.getParameter("pageSize").toString());
		
		int listCount = ProjScService.getProjectListCount(param);	
		
		int pageCount = (int) Math.ceil(listCount / pageSize);
		
		pageinMap.put("pageNo",pageNo);
		pageinMap.put("pageSize",pageSize);
		pageinMap.put("pageCount",pageCount);
		pageinMap.put("listCount",listCount);
		
		param.put("start", (int) (pageNo-1)*pageSize+1);
		param.put("end", (int) pageNo*pageSize);
		
		List list = ProjScService.getProjectList(param);	
 	  	model.put("list",list);
 	  	model.put("page",pageinMap);
 	  	
		return "jsonView";
	}
    
    /**
     * 프로젝트 작업별 일정
	 * @throws Exception 
     */
    @RequestMapping(value="/project/psView.do")
    public String psView(@RequestParam HashMap<Object,Object> param,HttpServletRequest request, Model model) throws Exception {

    	String projectid = request.getParameter("projectid");
    	
    	List list = ProjScService.getProjectUser(param);
    	
    	param.put("taskId", param.get("projectid"));
    	
    	List chargerList = ProjScService.projectChargerList(param);
    	
    	param.put("roleId", "SYSMANAGER");
		model.addAttribute("authority", CmmService.getAuthority(param));
		
    	model.addAttribute("regId", ProjScService.getProjectRegId(param));
    	
    	model.addAttribute("projectid", projectid);
    	model.addAttribute("list", list);
    	model.addAttribute("chargerList", chargerList);
    	
    	return "/dsdsm/project/psView";
    }
    
    /**
     * 프로젝트 프린트 페이지
	 * @throws Exception 
     */
    @RequestMapping(value="/project/psViewPrint.do")
    public String psViewPrint(@RequestParam HashMap<Object,Object> param, Model model) throws Exception {
    	
    	param.put("printarea1",param.get("printarea1").toString().replaceAll("\"", "'"));
    	param.put("printarea2",param.get("printarea2").toString().replaceAll("\"", "'"));
    	param.put("printarea3",param.get("printarea3").toString().replaceAll("\"", "'"));
    	
 	  	model.addAttribute("printarea1",param.get("printarea1"));
 	  	model.addAttribute("printarea2",param.get("printarea2"));
 	  	model.addAttribute("printarea3",param.get("printarea3"));
 	  	model.addAttribute("start",param.get("start1"));
 	  	model.addAttribute("end",param.get("end1"));
 	  	model.addAttribute("left1",param.get("left1"));
 	  	model.addAttribute("left2",param.get("left2"));
 	  	model.addAttribute("left3",param.get("left3"));
 	  	model.addAttribute("width1",param.get("width1"));
 	  	model.addAttribute("width2",param.get("width2"));
    	
    	return "/dsdsm/project/psViewPrint";
    }
    
    /**
     * 프로젝트 작업별 진척도
	 * @throws Exception 
     */
    @RequestMapping(value="/project/psProgressPop.do")
    public String psProgressPop(@RequestParam HashMap<Object,Object> param,HttpServletRequest request, Model model) throws Exception {
    
    	param.put("taskId", param.get("projectid"));
    	List<project> projectList = ProjScService.projectTaskList(param);
    	
    	SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd");
		
		for(int i=0;i<projectList.size();i++){
			Date start = simpleDateFormat.parse(projectList.get(i).getStarttime().toString());
			Date end = simpleDateFormat.parse(projectList.get(i).getEndtime().toString());
			String startdate = simpleDateFormat.format(start);
			String enddate = simpleDateFormat.format(end);
			projectList.get(i).setStartdate(startdate);
			projectList.get(i).setEnddate(enddate);
		}
		
    	List chargerList = ProjScService.projectChargerList(param);
    	
    	model.addAttribute("projectid", param.get("projectid"));
    	model.addAttribute("project",projectList);
    	model.addAttribute("chargerList", chargerList);
    	model.addAttribute("charger",param.get("searchVal"));
    	
    	return "/dsdsm/project/psProgressPop";
    }
    
    /**
     * 프로젝트 등록
	 * @throws Exception 
     */
    @RequestMapping(value="/project/psReg.do")
    public String psReg(HttpServletRequest request, Model model) throws Exception {
    	
    	return "/dsdsm/project/psReg";
    }
    
    /**
     * 프로젝트 수정
	 * @throws Exception 
     */
    @RequestMapping(value="/project/psEdit.do")
    public String psEdit(@RequestParam HashMap<Object,Object> param, Model model) throws Exception {
    	
    	List list = ProjScService.getProjectView(param);	
 	  	model.addAttribute("list",list);
    	
    	return "/dsdsm/project/psEdit";
    }
    
    @RequestMapping(value="/project/checkProjectid.json")
    public String checkProjectid(@RequestParam Map<String,String> param, ModelMap map) throws Exception {
    	
    	Map<String, Object> result = ProjScService.checkProjectid(param);  
    	
    	map.put("result", result);
    	
    	return "jsonView";
    	
    }
    
	@RequestMapping(value="/project/loadProject.json")
    public String loadProject(HttpSession session,@RequestParam HashMap<Object,Object> param,ModelMap model,tasks tasks,project project,assigs assgis,resources resources) throws Exception {
		
		List<project> projectList = ProjScService.projectTaskList(param);
		Map userInfo = (Map)session.getAttribute("userInfo");
		
		param.put("roleId", "SYSMANAGER");
		String authority = CmmService.getAuthority(param);
		
		for(int i=0;i<projectList.size();i++){
			param.put("projectid",param.get("taskId"));
			param.put("taskid",projectList.get(i).getId());
			projectList.get(i).setAssigs(ProjScService.projectAssigsList(param));
			
			param.put("userId", userInfo.get("userId"));
			String containUserId = ProjScService.getContainUserId(param);
			
			if(userInfo.get("userId").equals(projectList.get(i).getUserid()) || "Y".equals(authority)){
				projectList.get(i).setCanWrite(true);
			} else {
				if("1".equals(containUserId)){
					projectList.get(i).setCanWrite(true);
				} else {
					projectList.get(i).setCanWrite(false);
				}
			}
		}
		
		SimpleDateFormat simpleDateFormat = new SimpleDateFormat("yyyy-MM-dd",Locale.KOREA);
		
		for(int i=0;i<projectList.size();i++){
			Date start = simpleDateFormat.parse(projectList.get(i).getStarttime().toString());
			Date end = simpleDateFormat.parse(projectList.get(i).getEndtime().toString());
			
			projectList.get(i).setStart(start);
			projectList.get(i).setEnd(end);
		}
		
		List<project> templist = new ArrayList<project>();

		templist.addAll(projectList);
		
		tasks.setTasks(templist);
		
		if(param.get("searchVal") != null && !"".equals(param.get("searchVal"))){
			tasks.setCanWrite(false);
			tasks.setCanWriteOnParent(false);
		} else {
			tasks.setCanWrite(true);
			tasks.setCanWriteOnParent(true);
		}
		
		model.put("project",tasks);
		
   		return "jsonView";
    }
	
	@RequestMapping(value="/project/saveProject.json")
    public String saveProject(HttpServletRequest request,ModelMap map,HashMap<Object,Object> h,HashMap<Object,Object> h2,HashMap<Object,Object> h3) throws Exception {
		
		String[] deletelist = request.getParameterValues("deletelist");
		
		h.put("projectid", request.getParameter("projectid"));
		h2.put("projectid", request.getParameter("projectid"));
		h3.put("projectid", request.getParameter("projectid"));
		
		String jsonString = request.getParameter("prj");
		String escapedJson = StringEscapeUtils.unescapeHtml(jsonString);
				
		JSONTokener tokener = new JSONTokener( escapedJson );
		JSONObject jsonObject = new JSONObject( tokener );
		JSONArray list = (JSONArray) jsonObject.get("tasks");

		for(int i=0;i<list.length();i++){
			String escapedJson1 = StringEscapeUtils.unescapeHtml(list.get(i).toString());
			
			JSONTokener tokener1 = new JSONTokener( escapedJson1 );
			JSONObject jsonObject1 = new JSONObject( tokener1 );

			h.put("name", StringUtil.nullToEmpty(jsonObject1.get("name").toString()));
			h.put("progress", jsonObject1.get("progress").toString());
			h.put("description", StringUtil.nullToEmpty(jsonObject1.get("description").toString()));
			h.put("code", StringUtil.nullToEmpty(jsonObject1.get("code").toString()));
			h.put("level",jsonObject1.get("level").toString());
			h.put("status", StringUtil.nullToEmpty(jsonObject1.get("status").toString()));
			h.put("depends", StringUtil.nullToEmpty(jsonObject1.get("depends").toString()));
			h.put("canWrite", StringUtil.nullToEmpty(jsonObject1.get("canWrite").toString()));
			h.put("duration", jsonObject1.get("duration").toString());

			/* PostgreSQL DB에서 startIsMilestone, endIsMilestone 타입이 boolean 이어서 화면에서 null 값이 넘어온 경우 저장시 오류 발생하여 null 인 경우 false 로 셋팅함. */
			if(jsonObject1.isNull("startIsMilestone") || "".equals(jsonObject1.get("startIsMilestone"))){
				h.put("startIsMilestone", false);
			}else{
				h.put("startIsMilestone", jsonObject1.get("startIsMilestone"));
			}
			
			if(jsonObject1.isNull("endIsMilestone") || "".equals(jsonObject1.get("endIsMilestone"))){
				h.put("endIsMilestone", false);
			}else{
				h.put("endIsMilestone", jsonObject1.get("endIsMilestone"));
			}
			
			h.put("haschild", jsonObject1.get("hasChild"));
			
			Date startDay = new Date ( (Long) jsonObject1.get("start") );
	    	DateFormat startFormat = DateFormat.getDateInstance ( DateFormat.DEFAULT,Locale.KOREA );
	    	String start = startFormat.format ( startDay );
	    	
	    	Date endDay = new Date ( (Long) jsonObject1.get("end") );
	    	DateFormat endFormat = DateFormat.getDateInstance ( DateFormat.DEFAULT,Locale.KOREA );
	    	String end = endFormat.format ( endDay );
	    	
	    	h.put("start",start);
	    	h.put("end", end);
	    	h.put("sortorder", i);
	    	
			if(jsonObject1.get("id").toString().contains("tmp_")){
				//신규
				System.out.println("★★★★★ 신규 ★★★★★");
				
				h.put("id", ProjScService.getTaskid(h));
				
				ProjScService.regTasks(h);
			} else {
				//수정
				System.out.println("★★★★★ 수정 ★★★★★");
				
				h.put("id", jsonObject1.get("id"));

				ProjScService.updateTasks(h);
			}
			
		}
		
		int deletelength = Integer.parseInt(request.getParameter("deletelength"));
		
		if(deletelength > 0){
			//삭제
			System.out.println("★★★★★ 삭제 ★★★★★");
			
		    h.put("deleteid", deletelist);
			ProjScService.deleteTasks(h);
		}
		
   		return "jsonView";
    }
	
	/**
     * 프로젝트 편집
     * @throws Exception 
     */
    @RequestMapping(value="/project/editProject.json")
    public String editProject(@RequestParam HashMap<Object,Object> param,HttpServletRequest request,ModelMap map) throws Exception {
    	
    	Map<String, Object> result = null;
    	System.out.println(param.get("gbn"));
    	if("S".equals(param.get("gbn"))){
    		result = ProjScService.regProject(param);
    	} else if("U".equals(param.get("gbn"))){
    		result = ProjScService.saveProject(param);
    	} else if("D".equals(param.get("gbn"))){
    		result = ProjScService.delProject(param);
    	}
    	
    	map.put("result", result);
    	map.put("projectid", param.get("projectid"));
    	
    	return "jsonView";
    }
    
    /**
     * 담당자 그룹 등록
     * @throws Exception 
     */
    @RequestMapping(value="/project/addChargerGroup.json")
    public String addChargerGroup(@RequestParam HashMap<Object,Object> param,HttpServletRequest request,ModelMap map) throws Exception {
    	
    	Map<String, Object> result = null;
    	
    	if("P".equals(param.get("type"))){
    		result = ProjScService.addCharger(param); 
    	} else if("G".equals(param.get("type"))){
    		result = ProjScService.addChargerGroup(param);
    	} else if("D".equals(param.get("type"))){
    		result = ProjScService.delCharger(param);
    		ProjScService.delTaskCharger(param);
    	}
    	
    	map.put("result", result);
    	
    	return "jsonView";
    }
    
    /**
     * 작업별 담당자 등록
     * @throws Exception 
     */
    @RequestMapping(value="/project/addTaskCharger.json")
    public String addTaskCharger(@RequestParam("nodearr") String[] nodeArr,@RequestParam HashMap<Object,Object> param,HttpServletRequest request,ModelMap map) throws Exception {
    	
    	Map<String, Object> result = null;
    	@SuppressWarnings("rawtypes")
		List<HashMap> checkUpperTask = null;
    	List<HashMap> checkLowerTask = null;
    	
    	for(int j=0;j<nodeArr.length;j++){
    		System.out.println("★★★★ " + nodeArr[j]);
    		param.put("nodeId", nodeArr[j]);

    		
	    	if("S".equals(param.get("type"))){
	    		result = ProjScService.addTaskCharger(param); 
	    		
	    		checkUpperTask = ProjScService.checkUpperTask(param);
	    		
	    		for(int i=0;i<checkUpperTask.size();i++){
	    			param.put("taskid", checkUpperTask.get(i).get("taskid"));
	    			ProjScService.addTaskCharger(param);
	    		}
	    	} else if("D".equals(param.get("type"))){
	    		result = ProjScService.delTaskCharger(param);
	    		
	    		checkLowerTask = ProjScService.checkLowerTask(param);
	    		
	    		for(int i=0;i<checkLowerTask.size();i++){
	    			param.put("taskid", checkLowerTask.get(i).get("taskid"));
	    			ProjScService.delTaskCharger(param);
	    		}
	    	}
    		
    	}
    	
    	map.put("result", result);
    	
    	return "jsonView";
    }
	
    /**
     * 그룹 추가
	 * @throws Exception 
     */
    @RequestMapping(value="/project/psAddGroup.do")
    public String psAddGroup(HttpServletRequest request, Model model) throws Exception {
    	model.addAttribute("projectid", (String) request.getParameter("projectid"));
    	return "/dsdsm/project/psAddGroup";
    }
    
    /**
     * 담당자 추가
	 * @throws Exception 
     */
    @RequestMapping(value="/project/psAddCharger.do")
    public String psAddCharger(HttpServletRequest request, Model model) throws Exception {
    	@SuppressWarnings("rawtypes")
		List deptCodes = commonDataService.getDeptCode();
    	
    	ObjectMapper mapper = new ObjectMapper();
   		String deptCodesJsonStr = mapper.writeValueAsString(deptCodes);
    	
    	model.addAttribute("DEPT_CODE", deptCodesJsonStr);
    	model.addAttribute("projectid", (String) request.getParameter("projectid"));

    	return "/dsdsm/project/psAddCharger";
    }
}
