package egovframework.rte.dsdsm.sr.service;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import egovframework.rte.dsdsm.cmm.service.vo.usrVO;
import egovframework.rte.psl.dataaccess.util.EgovMap;

public interface SrService {
	
	/**
	 * S/R 번호 생성하기
	 * @return int
	 * @exception Exception 
	*/
	public int getSrno(HashMap h) throws Exception;
	
	/**
	 * 마지막단계 권한
	 * @return String
	 * @exception Exception 
	*/
	public String getRole(HashMap h) throws Exception;
	
	/**
	 * 문서에서 넘어오는 경우(관련공문명 가져오기)
	 * @return String
	 * @exception Exception 
	*/
	public String getRelText(HashMap h) throws Exception;
	
	/**
	 * S/R HISTORY 순번 생성하기
	 * @return int
	 * @exception Exception 
	*/
	public int getSrseq(HashMap h) throws Exception;
	
	/**
	 * S/R 업무변경 HISTORY 순번 생성하기
	 * @return int
	 * @exception Exception 
	*/
	public int getSrJobseq(Map<String,String> h) throws Exception;
	
	/**
	 * S/R 접수여부
	 * @return String
	 * @exception Exception 
	*/
	public String getReceiptYn(HashMap h) throws Exception;
	
	/**
	 * S/R 등록
	 * @return int
	 * @exception Exception 
	*/
	public Map<String,Object> regSrmain(HashMap h) throws Exception;
	
	/**
	 * S/R 등록
	 * @return int
	 * @exception Exception 
	*/
	public Map<String,Object> regHistory(HashMap h) throws Exception;
	
	/**
	 * S/R 업무코드 변경이력 등록
	 * @return int
	 * @exception Exception 
	*/
	public Map<String,Object> regJobHistory(Map<String,String> h) throws Exception;
	
	/**
	 * S/R 목록을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return S/R 목록
	 * @exception Exception
	 */
    public List getList(HashMap h) throws Exception;
    
    /**
	 * S/R 목록을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return S/R 목록
	 * @exception Exception
	 */
    public List getView(HashMap h) throws Exception;

    /**
	 * S/R 파일 업로드 목록을 조회한다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return S/R 파일 업로드 목록
	 * @exception Exception
	 */
    public List getFileUploadList(HashMap h) throws Exception;
    
    /**
	 * S/R 히스토리 상태 목록
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return S/R 목록
	 * @exception Exception
	 */
    public List getStatus(HashMap h) throws Exception;
    
    /**
	 * S/R 단계등록
	 * @return int
	 * @exception Exception 
	*/
	public Map<String,Object> updateAccept(HashMap h) throws Exception;
	
	/**
	 * S/R 최종단계등록
	 * @return int
	 * @exception Exception 
	*/
	public int updateLaststa(HashMap h) throws Exception;

	/**
	 * S/R 최종단계 반려여부
	 * @return int
	 * @exception Exception 
	*/
	public int updateSrReturnYn(HashMap h) throws Exception;
	
	/**
	 * S/R 부서입력
	 * @return int
	 * @exception Exception 
	*/
	public int updateSrDeptId(HashMap h) throws Exception;
	
	/**
	 * S/R 총 갯수
	 * @return int
	 * @exception Exception 
	*/
	public int getListCount(HashMap h) throws Exception;
	
	/**
	 * S/R 메인 마지막단계 불러오기
	 * @return List
	 * @exception Exception 
	*/
	public List getLaststa(HashMap h) throws Exception;
	
	/**
	 * S/R 히스토리 마지막단계 불러오기
	 * @return List
	 * @exception Exception 
	*/
	public List getHistorysta(HashMap h) throws Exception;
	
	/**
	 * S/R 저장
	 * @return int
	 * @exception Exception 
	*/
	public String insertContent(HashMap h) throws Exception;
	
	/**
	 * S/R 수정
	 * @return int
	 * @exception Exception 
	*/
	public void updateContent(HashMap h) throws Exception;
	
	/**
	 * S/R HISTORY 수정
	 * @return int
	 * @exception Exception 
	*/
	public void updateHistory(HashMap h) throws Exception;
	
	/**
	 * S/R 상세내용을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return S/R 상세내용
	 * @exception Exception
	 */
    public List getSRdetail(HashMap h) throws Exception;
    
    /**
	 * S/R 부서 구성원 목록을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return S/R 부서 구성원 목록
	 * @exception Exception
	 */
    public List getUserCode(HashMap h) throws Exception;
    
    public List selectAtchFileList(HashMap map) throws Exception;
    
    
    public List selectSrProcStep() throws Exception;
}