package egovframework.rte.dsdsm.taskOrder.service.impl;

import java.sql.Clob;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.springframework.stereotype.Repository;

import egovframework.rte.cmmn.util.EgovProperties;
import egovframework.rte.dsdsm.cmm.util.StrUtil;
import egovframework.rte.psl.dataaccess.EgovAbstractDAO;
import egovframework.rte.psl.dataaccess.util.EgovMap;

@Repository("taskOrderDAO")
public class taskOrderDAO extends EgovAbstractDAO {
	private static String databaseType = StrUtil.NVL(EgovProperties.getProperty("Globals.DatabaseType"),"postgres");
	
    /**
	 * 
	 * @return 사업 총 갯수
	 * @exception Exception 
	*/
	public int getBusinessListCount(HashMap h) throws Exception {
		return (Integer) select("taskOrderDAO.getBusinessListCnt", h);		
	} 
	
	/**
	 * 
	 * @return 사업 목록
	 * @exception Exception 
	*/
	public List getBusinessList(HashMap h) throws Exception {
		List list = list("taskOrderDAO.getBusinessList", h);
		
		// databaseType이 oracle인 경우 Clob 데이터 처리
		if("oracle".equals(databaseType)){
			for(int i = 0; i < list.size(); i++){
				// Clob 데이터를 String 으로 변환 해주지 않으면.. json type로 변환시 에러남
				// 이에 clob 데이터를 String으로 전환하여 Map에 다시 input
				EgovMap dataMap = (EgovMap)list.get(i);
				Clob data = (Clob)dataMap.get("bnsBeego");
				String clobStr = StrUtil.clobToString(data);
				dataMap.put("bnsBeego", clobStr);
			}
		}
		
		return list;
	}
	
	
	/**
	 * 
	 * @return 사업등록 체크 
	 * @exception Exception 
	*/
	public int businessCheckCount(HashMap h){
		return (Integer) select("taskOrderDAO.getBusinessCheckCount", h);
	}	
	
	
	
	/**
	 * 
	 * @return 사업ID 중복체크
	 * @exception Exception 
	*/
	public Map<String,Object> checkBusinessid(Map<String,String> map){
		Map<String, Object> result = new HashMap<String,Object>();
		result.put("ONE", select("taskOrderDAO.checkBusinessid", map));
		return result;
	}
	
	/**
	 * 
	 * @return 사업 등록
	 * @exception Exception 
	*/
	public Map<String,Object> regBusiness(HashMap h){
		Map<String, Object> result = new HashMap<String,Object>();
		result.put("CNT", insert("taskOrderDAO.regBusiness", h));
		return result;
	}	
	
	/**
	 * 
	 * @return 사업 세부내용
	 * @exception Exception 
	*/
	public List getBusinessView(HashMap h) throws Exception {
		List list = list("taskOrderDAO.getBusinessView", h);
		
		// databaseType이 oracle인 경우 Clob 데이터 처리
		if("oracle".equals(databaseType)){
			for(int i = 0; i < list.size(); i++){
				// Clob 데이터를 String 으로 변환 해주지 않으면.. json type로 변환시 에러남
				// 이에 clob 데이터를 String으로 전환하여 Map에 다시 input
				EgovMap dataMap = (EgovMap)list.get(i);
				System.out.println("dataMap : " + dataMap.toString());
				Clob data = (Clob)dataMap.get("businessbeego");
				String clobStr = StrUtil.clobToString(data);
				dataMap.put("businessbeego", clobStr);
			}
		}
		
		return list;
	}	
		
	
	/**
	 * 
	 * @return 사업 수정
	 * @exception Exception 
	*/
	public Map<String,Object> editBusiness(HashMap h){
		Map<String, Object> result = new HashMap<String,Object>();
		result.put("CNT", insert("taskOrderDAO.editBusiness", h));
		return result;
	}	
	
	
	/**
	 * 
	 * @return 사업 삭제
	 * @exception Exception 
	*/
	public Map<String,Object> delBusiness(HashMap h){
		Map<String, Object> result = new HashMap<String,Object>();
		result.put("CNT", insert("taskOrderDAO.delBusiness", h));
		return result;
	}
	
	
	
    /**
	 * 
	 * @return 사업 총 갯수
	 * @exception Exception 
	*/
	public int getTaskListCount(HashMap h) throws Exception {
		return (Integer) select("taskOrderDAO.getBusinessTaskListCount", h);		
	} 
	
	/**
	 * 
	 * @return 사업 목록
	 * @exception Exception 
	*/
	public List getTaskList(HashMap h) throws Exception {
		List list = list("taskOrderDAO.getBusinessTaskList", h);
		
		// databaseType이 oracle인 경우 Clob 데이터 처리
		if("oracle".equals(databaseType)){
			for(int i = 0; i < list.size(); i++){
				// Clob 데이터를 String 으로 변환 해주지 않으면.. json type로 변환시 에러남
				// 이에 clob 데이터를 String으로 전환하여 Map에 다시 input
				EgovMap dataMap = (EgovMap)list.get(i);
				Clob data = (Clob)dataMap.get("businessbeego");
				String clobStr = StrUtil.clobToString(data);
				dataMap.put("businessbeego", clobStr);
			}
		}
		
		return list;
	}	

	
	/**
	 * 
	 * @return 사업 상세보기
	 * @exception Exception 
	*/
	public List getBusinessTaskView(HashMap h) throws Exception {
		return list("taskOrderDAO.getBusinessTaskView", h);
	}	
	
	
	/**
	 * 
	 * @return 사업_과업 상세보기
	 * @exception Exception 
	*/
	public List getBusinessTaskView2(HashMap h) throws Exception {
		List list = list("taskOrderDAO.getBusinessTaskView2", h);
		
		// databaseType이 oracle인 경우 Clob 데이터 처리
		if("oracle".equals(databaseType)){
			for(int i = 0; i < list.size(); i++){
				// Clob 데이터를 String 으로 변환 해주지 않으면.. json type로 변환시 에러남
				// 이에 clob 데이터를 String으로 전환하여 Map에 다시 input
				EgovMap dataMap = (EgovMap)list.get(i);
				
				Clob reqDetailsDescription = (Clob)dataMap.get("reqDetailsDescription");
				String clobDescriptionStr = StrUtil.clobToString(reqDetailsDescription);
				dataMap.put("reqDetailsDescription", clobDescriptionStr);
				
				Clob reqAcceptable = (Clob)dataMap.get("reqAcceptable");
				String clobAcceptableStr = StrUtil.clobToString(reqAcceptable);
				dataMap.put("reqAcceptable", clobAcceptableStr);
			}
		}
		
		return list;
	}	
	
	
	/**
	 * 
	 * @return 문서 총 갯수(popup)
	 * @exception Exception 
	*/
	public int getPopupListCount(HashMap h) throws Exception {
		return (Integer) select("taskOrderDAO.getBusinessListCnt", h);
		                         
	}	
	
	/**
	 * 
	 * @return 문서목록(popup)
	 * @exception Exception 
	*/
	public List getPopupList(HashMap h) throws Exception {
		List list = list("taskOrderDAO.getBusinessList", h);
		
		// databaseType이 oracle인 경우 Clob 데이터 처리
		if("oracle".equals(databaseType)){
			for(int i = 0; i < list.size(); i++){
				// Clob 데이터를 String 으로 변환 해주지 않으면.. json type로 변환시 에러남
				// 이에 clob 데이터를 String으로 전환하여 Map에 다시 input
				EgovMap dataMap = (EgovMap)list.get(i);
				Clob data = (Clob)dataMap.get("bnsBeego");
				String clobStr = StrUtil.clobToString(data);
				dataMap.put("bnsBeego", clobStr);
			}
		}
		
		return list;
	}		
	
	/**
	 * 
	 * @return 사업 등록
	 * @exception Exception 
	*/
	public Map<String,Object> regTask(HashMap h){
		Map<String, Object> result = new HashMap<String,Object>();
		result.put("taskSno", insert("taskOrderDAO.regTask", h));
		return result;
	}	
	
	/**
	 * 
	 * @return 과업등록 체크 
	 * @exception Exception 
	*/
	public int taskCheckCount(HashMap h){
		return (Integer) select("taskOrderDAO.getTaskCheckCount", h);
	}	
	
	

	
	
	
	
	
	/**
	 * 
	 * @return 과업정보 등록
	 * @exception Exception 
	*/
	public Map<String,Object> taskUpdate(HashMap h){
		Map<String, Object> result = new HashMap<String,Object>();
		result.put("CNT", insert("taskOrderDAO.taskUpdate", h));
		return result;
	}		
	
	
	/**
	 * 
	 * @return 과업정보 삭제
	 * @exception Exception 
	*/
	public Map<String,Object> taskDelete(HashMap h){
		Map<String, Object> result = new HashMap<String,Object>();
		result.put("CNT", insert("taskOrderDAO.taskDelete", h));
		return result;
	}	
	
	

	/**
	 * Project Task 총 갯수
	 * @return int
	 * @exception Exception 
	*/
    public int getPtListCount(HashMap h) throws Exception {
    	return (Integer) select("taskOrderDAO.getBusinessListCnt", h);
	}
    
    
    
	/**
	 * Project Task  목록을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 프로잭트와 세부일정
	 * @exception Exception
	 */
    public List getPtList(HashMap h) throws Exception {
    	return list("taskOrderDAO.getPopupProejectTask", h);
    } 	
    
	/**
	 * 과업에대한 task 현황 등록 및 수정
	 * @return 과업에대한 task 현황 등록 및 수정
	 * @exception Exception 
	*/
	public Map<String,Object> taskSituationDelete(HashMap h){
		Map<String, Object> result = new HashMap<String,Object>();
		result.put("CNT", insert("taskOrderDAO.taskProgressdelete", h));
		return result;
	}	    
	
	
	/**
	 * 과업에대한 task 현황 등록 및 수정
	 * @return 과업에대한 task 현황 등록 및 수정
	 * @exception Exception 
	*/
	public Map<String,Object> taskSituation(HashMap h){
		Map<String, Object> result = new HashMap<String,Object>();
		result.put("CNT", insert("taskOrderDAO.taskSituation", h));
		return result;
	}	
	
	
	/**
	 * 
	 * @return 사업_과업 상세보기
	 * @exception Exception 
	*/
	public List getProjectTaskView(HashMap h) throws Exception {
		return list("taskOrderDAO.getProjectTaskList", h);
	}		
	
	/**
	 * 
	 * @return 산출물 목록정보 
	 * @exception Exception 
	*/
	public List getScmList(HashMap h) throws Exception {
		return list("taskOrderDAO.getSabchulmoolList", h);
	}
	
	
	/**
	 * 
	 * @return 사업_과업 상세보기
	 * @exception Exception 
	*/
	public List getSanchurmulView(HashMap h) throws Exception {
		return list("taskOrderDAO.taskSanchurmulList", h);
	}	
	
	/**
	 * 과업에대한 산출물  현황 등록 및 수정
	 * @return 과업에대한 task 현황 등록 및 수정
	 * @exception Exception 
	*/
	public Map<String,Object> taskSituationScmDelete(HashMap h){
		Map<String, Object> result = new HashMap<String,Object>();
		result.put("CNT", insert("taskOrderDAO.taskSanchurmulDelete", h));
		return result;
	}		
	
	
	/**
	 * 과업에대한 task 현황 등록 및 수정
	 * @return 과업에대한 task 현황 등록 및 수정
	 * @exception Exception 
	*/
	public Map<String,Object> taskSituationScm(HashMap h){
		Map<String, Object> result = new HashMap<String,Object>();
		result.put("CNT", insert("taskOrderDAO.taskSanchurmul", h));
		return result;
	}
	
	
	/**
	 * 요구사항  목록 정보(Popup) 총 갯수
	 * @return int
	 * @exception Exception 
	*/
    public int getTskOdrListPopCnt(HashMap h) throws Exception {
    	//return (Integer) select("taskOrderDAO.getTskOdrListPopCnt", h);
    	return (Integer) select("taskOrderDAO.getBusinessTaskListCount", h);
	}
    
    
	/**
	 * 요구사항  목록 정보(Popup)
	 * @return int
	 * @exception Exception 
	*/
    public List getTskOdrListPop(HashMap h) throws Exception {
    	//return list("taskOrderDAO.getTskOdrListPop", h);
    	return list("taskOrderDAO.getBusinessTaskList", h);
	}
	
	
}
