package egovframework.rte.dsdsm.taskOrder.service.impl;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Service;

import egovframework.rte.fdl.cmmn.AbstractServiceImpl;
import egovframework.rte.fdl.cmmn.EgovAbstractServiceImpl;
import egovframework.rte.dsdsm.projectSchedule.service.ProjScService;
import egovframework.rte.dsdsm.taskOrder.service.taskOrderService;


@Service("taskOrderService")
public class taskOrderimpl extends EgovAbstractServiceImpl implements taskOrderService{	
	
    @Resource(name="taskOrderDAO")
    private taskOrderDAO taskOrderDAO;
    
	private Logger log = LogManager.getLogger();
	
	
	/**
	 * 사업과업 총 갯수
	 * @exception Exception
	 */
    public int getBusinessListCount(HashMap h) throws Exception {
		return taskOrderDAO.getBusinessListCount(h);
	}
    
    /**
	 * 사업과업 목록
	 * @exception Exception
	 */
    public List getBusinessList(HashMap h) throws Exception {
        return taskOrderDAO.getBusinessList(h);
    }

    
    /**
   	 * 사업ID 중복체크
   	 * @exception Exception
   	 */
    @Override
	public Map<String, Object> checkBusinessid(Map<String, String> map) throws Exception {
		
		Map <String,Object> result = taskOrderDAO.checkBusinessid(map);
		
		@SuppressWarnings("unchecked")
		Map<String, Object> one = (Map<String, Object>)result.get("ONE");
		
		if(one==null){
			result.put("SUCCESS", true);
			result.put("MESSAGE", "사업ID를 사용할 수 있습니다.");
		}else{
			result.put("SUCCESS", false);
			result.put("MESSAGE", "사업ID가 중복되었습니다.<br>다른 ID를 입력해주세요.");
		}
		
		return result;
	}  
    
    /**
   	 * 사업ID 등록
   	 * @exception Exception
   	 */
    @Override
	public Map<String, Object> regBusiness(HashMap h) throws Exception {    
    	
		Map <String,Object> bnsSnoInfo = taskOrderDAO.regBusiness(h);
		Map <String,Object> result = new HashMap<String, Object>();
		
		
		String businessSno = (String)bnsSnoInfo.get("CNT");
		
		log.debug(bnsSnoInfo);
		HashMap<String, String> h2 = new HashMap<String, String>();
		h2.put("businessSno", businessSno);
		
		int cnt  = taskOrderDAO.businessCheckCount(h2);
		 
		if(cnt > 0){
			result.put("SUCCESS", true);
			result.put("businessSno", businessSno);
		}else{
			result.put("SUCCESS", false);
		}
		
		
		return result;
	}

    
    /**
	 * 프로젝트 세부내용
	 * @exception Exception
	 */
    public List getBusinessView(HashMap h) throws Exception {
        return taskOrderDAO.getBusinessView(h); 
    }
    
    
    /**
   	 * 사업ID 수정
   	 * @exception Exception
   	 */    
    @Override
	public Map<String, Object> editBusiness(HashMap h) throws Exception {    
    	
		Map <String,Object> result = taskOrderDAO.editBusiness(h);
		
		String cnt = (String)result.get("CNT");
		
		if(cnt==null){
			result.put("SUCCESS", true);
		}else{
			result.put("SUCCESS", false);
		}
		
		return result;
	}
    
    
    
    /**
   	 * 사업ID 삭제
   	 * @exception Exception
   	 */    
    @Override
	public Map<String, Object> delBusiness(HashMap h) throws Exception {    
    	
		Map <String,Object> result = taskOrderDAO.delBusiness(h);
		
		String cnt = (String)result.get("CNT");
		
		if(cnt==null){
			result.put("SUCCESS", true);
		}else{
			result.put("SUCCESS", false);
		}
		
		return result;
	}    
    
    
    /**
	 * 사업과업 목록
	 * @exception Exception
	 */
    public List getBusinessTaskView(HashMap h) throws Exception {
    	
    	List rtnList = null;
    	String taskSno = (String)h.get("taskSno");
    	
    	if(taskSno.length() < 32){
    		rtnList = taskOrderDAO.getBusinessTaskView(h);
    	} else {
    		rtnList = taskOrderDAO.getBusinessTaskView2(h);
    	}
    	
    	
        return rtnList;
    }    
        
    
    /**
	 * 문서 총 갯수(popup)
	 * @exception Exception
	 */
    public int getPopupListCount(HashMap h) throws Exception {
		return taskOrderDAO.getPopupListCount(h);
	} 
    
    /**
	 * 문서목록(popup)
	 * @exception Exception
	 */
    public List getPopupList(HashMap h) throws Exception {
        return taskOrderDAO.getPopupList(h);
    }
    
    
    
	/**
	 * 사업과업 총 갯수
	 * @exception Exception
	 */
    public int getTaskListCount(HashMap h) throws Exception {
		return taskOrderDAO.getTaskListCount(h);
	}
    
    /**
	 * 사업과업 목록
	 * @exception Exception
	 */
    public List getTaskList(HashMap h) throws Exception {
        return taskOrderDAO.getTaskList(h);
    }    
    
    
    /**
	 * 과업사항 테이블에 등록
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 과업사항 테이블에 등록
	 * @exception Exception
	 */    
    public Map<String, Object> regTask(HashMap h) throws Exception {    
    	

		Map <String,Object> taskSnoInfo = taskOrderDAO.regTask(h);
		Map <String,Object> result = new HashMap<String, Object>();
		
		String taskSno = (String)taskSnoInfo.get("taskSno");
		
		log.debug(taskSno);
		HashMap<String, String> h2 = new HashMap<String, String>();
		h2.put("taskSno", taskSno);
		
		int cnt  = taskOrderDAO.taskCheckCount(h2);
		 
		if(cnt > 0){
			result.put("SUCCESS", true);
			result.put("taskSno", taskSno);
		}else{
			result.put("SUCCESS", false);
		}
		
		return result;
     }
    
    
    /**
	 * 과업정보 테이블에 수정
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 과업사항 테이블에 등록
	 * @exception Exception
	 */    
    public Map<String, Object> taskUpdate(HashMap h) throws Exception {    
    	
		Map <String,Object> result = taskOrderDAO.taskUpdate(h);
		
		String cnt = (String)result.get("CNT");
		
		if(cnt==null){
			result.put("SUCCESS", true);
		}else{
			result.put("SUCCESS", false);
		}
		
		return result;
     } 
    
    
    /**
	 * 과업정보 테이블에 삭제
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 과업사항 테이블에 등록
	 * @exception Exception
	 */    
    @Override
	public Map<String, Object> taskDelete(HashMap h) throws Exception {    
    	
		Map <String,Object> result = taskOrderDAO.taskDelete(h);
		
		String cnt = (String)result.get("CNT");
		
		if(cnt==null){
			result.put("SUCCESS", true);
		}else{
			result.put("SUCCESS", false);
		}
		
		return result;
	}     
    
    
	/**
	 * Project Task 총 갯수
	 * @return int
	 * @exception Exception 
	*/
    public int getPtListCount(HashMap h) throws Exception {
		return taskOrderDAO.getPtListCount(h);
	}
    
    
	/**
	 * Project Task  목록을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 프로잭트와 세부일정
	 * @exception Exception
	 */
    public List getPtList(HashMap h) throws Exception {
        return taskOrderDAO.getPtList(h);
    }    

    /**
	 * 과업에대한 task 현황 등록 및 수정
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 과업사항 테이블에 등록
	 * @exception Exception
	 */    
    public Map<String, Object> taskSituationDelete(HashMap h) throws Exception {    
    	
		Map <String,Object> result = taskOrderDAO.taskSituationDelete(h);
		
		String cnt = (String)result.get("CNT");
		
		if(cnt==null){
			result.put("SUCCESS", true);
		}else{
			result.put("SUCCESS", false);
		}
		
		return result;
     }  
    
    /**
	 * 과업에대한 task 현황 등록 및 수정
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 과업사항 테이블에 등록
	 * @exception Exception
	 */    
    public Map<String, Object> taskSituation(HashMap h) throws Exception {    
    	
		Map <String,Object> result = taskOrderDAO.taskSituation(h);
		
		String cnt = (String)result.get("CNT");
		
		if(cnt==null){
			result.put("SUCCESS", true);
		}else{
			result.put("SUCCESS", false);
		}
		
		return result;
     } 
    
    /**
	 * 사업과업 목록
	 * @exception Exception
	 */
    public List getProjectTaskView(HashMap h) throws Exception {
    	
    	List rtnList = null;
    	String taskSno = (String)h.get("taskSno");
    	
    	rtnList = taskOrderDAO.getProjectTaskView(h);
    	
        return rtnList;
    }   
    
    
    /**
	 * 산출물 목록정보 
	 * @exception Exception
	 */
    public List getScmList(HashMap h) throws Exception {
        return taskOrderDAO.getScmList(h);
    }   
    
    
    /**
	 * 과업에대한 산출물 목록
	 * @exception Exception
	 */
    public List getSanchurmulView(HashMap h) throws Exception {
    	
    	List rtnList = null;
    	String taskSno = (String)h.get("taskSno");
    	
    	rtnList = taskOrderDAO.getSanchurmulView(h);
    	
        return rtnList;
    }       
    
    
    /**
	 * 과업에대한 산출물  현황 등록 및 수정
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 과업사항 테이블에 등록
	 * @exception Exception
	 */    
    public Map<String, Object> taskSituationScmDelete(HashMap h) throws Exception {    
    	
		Map <String,Object> result = taskOrderDAO.taskSituationScmDelete(h);
		
		String cnt = (String)result.get("CNT");
		
		if(cnt==null){
			result.put("SUCCESS", true);
		}else{
			result.put("SUCCESS", false);
		}
		
		return result;
     }   
    
    /**
	 * 과업에대한 task 현황 등록 및 수정
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 과업사항 테이블에 등록
	 * @exception Exception
	 */    
    public Map<String, Object> taskSituationScm(HashMap h) throws Exception {    
    	
		Map <String,Object> result = taskOrderDAO.taskSituationScm(h);
		
		String cnt = (String)result.get("CNT");
		
		if(cnt==null){
			result.put("SUCCESS", true);
		}else{
			result.put("SUCCESS", false);
		}
		
		return result;
     }
    
    
	/**
	 * 요구사항  목록 정보(Popup) 총 갯수
	 * @exception Exception
	 */
    public int getTskOdrListPopCnt(HashMap h) throws Exception {
		return taskOrderDAO.getTskOdrListPopCnt(h);
	}    
    
	/**
	 * 요구사항  목록 정보(Popup)
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return요구사항  목록 정보
	 * @exception Exception
	 */
    public List getTskOdrListPop(HashMap h) throws Exception {
        return taskOrderDAO.getTskOdrListPop(h);
    }       
}
