package egovframework.rte.dsdsm.taskOrder.service;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import egovframework.rte.psl.dataaccess.util.EgovMap;


public interface taskOrderService {	
	
	
	/**
	 * 사업과업 총 갯수
	 * @return int
	 * @exception Exception 
	*/
	public int getBusinessListCount(HashMap h) throws Exception;
	
	
	/**
	 * 사업과업 목록을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 사업과업 목록
	 * @exception Exception
	 */
    public List getBusinessList(HashMap h) throws Exception;
    
    
    
    /**
	 * 사업ID 중복체크
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 사업ID 중복체크
	 * @exception Exception
	 */
    public Map<String,Object> checkBusinessid(Map<String, String> map) throws Exception;
    
    
    /**
	 * 사업ID 등록
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 사업 테이블에 등록
	 * @exception Exception
	 */    
    public Map<String,Object> regBusiness(HashMap h) throws Exception;
    
    
    /**
	 * 사업ID 등록정보
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 사업ID 관련 정보 조회
	 * @exception Exception
	 */      
    public List getBusinessView(HashMap h)throws Exception;
    
    
    /**
	 * 사업ID 수정
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 사업 테이블에 수정
	 * @exception Exception
	 */    
    public Map<String,Object> editBusiness(HashMap h) throws Exception;    
    
    
    
    /**
	 * 사업ID 삭제
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 사업 테이블에 수정
	 * @exception Exception
	 */    
    public Map<String,Object> delBusiness(HashMap h) throws Exception;        
    
    
    
	/**
	 * 사업과업 상세화면을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 사업과업 상세
	 * @exception Exception
	 */
    public List getBusinessTaskView(HashMap h) throws Exception;
    
    
	/**
	 * 문서 총 갯수(popup)
	 * @return int
	 * @exception Exception 
	*/
	public int getPopupListCount(HashMap h) throws Exception;  
    
    
	/**
	 * 문서 목록을 불러온다(popup)
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return S/R 목록
	 * @exception Exception
	 */
    public List getPopupList(HashMap h) throws Exception;	
    
    
    
	/**
	 * 과업정보 총 갯수
	 * @return int
	 * @exception Exception 
	*/
	public int getTaskListCount(HashMap h) throws Exception;
	
	
	/**
	 * 과업정보 목록을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 사업과업 목록
	 * @exception Exception
	 */
    public List getTaskList(HashMap h) throws Exception;    
    
    
    
    
    /**
	 * 과업사항 테이블에 등록
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 과업사항 테이블에 등록
	 * @exception Exception
	 */    
    public Map<String,Object> regTask(HashMap h) throws Exception;  
    
    
    /**
	 * 과업사항 테이블에 수정
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 과업사항 테이블에 수정
	 * @exception Exception
	 */    
    public Map<String,Object> taskUpdate(HashMap h) throws Exception;   
    
    
    /**
	 * 과업정보 삭제
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 사업 테이블에 수정
	 * @exception Exception
	 */    
    public Map<String,Object> taskDelete(HashMap h) throws Exception;  
    
    

	/**
	 * Project Task 총 갯수
	 * @return int
	 * @exception Exception 
	*/
	public int getPtListCount(HashMap h) throws Exception;
	
	
	/**
	 * Project Task  목록을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 프로잭트와 세부일정
	 * @exception Exception
	 */
    public List getPtList(HashMap h) throws Exception; 
    
    
	/**
	 * 과업에대한 task 현황 등록 및 수정
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 프로잭트와 세부일정
	 * @exception Exception
	 */
    public Map<String,Object> taskSituationDelete(HashMap h) throws Exception;       
    

	/**
	 * 과업에대한 task 현황 등록 및 수정
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 프로잭트와 세부일정
	 * @exception Exception
	 */
    public Map<String,Object> taskSituation(HashMap h) throws Exception;     
    
    
	/**
	 * 사업과업 상세화면을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 사업과업 상세
	 * @exception Exception
	 */
    public List getProjectTaskView(HashMap h) throws Exception;    

    
	/**
	 * 산출물 목록정보 
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 산출물 목록정보 
	 * @exception Exception
	 */
    public List getScmList(HashMap h) throws Exception;     
    
    
	/**
	 * 사업과업 상세화면을 불러온다
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 사업과업 상세
	 * @exception Exception
	 */
    public List getSanchurmulView(HashMap h) throws Exception;   
    
    
	/**
	 * 과업에대한 산출물  현황 등록 및 수정
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 프로잭트와 세부일정
	 * @exception Exception
	 */
    public Map<String,Object> taskSituationScmDelete(HashMap h) throws Exception;       
        
    
	/**
	 * 과업에대한 task 현황 등록 및 수정
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 프로잭트와 세부일정
	 * @exception Exception
	 */
    public Map<String,Object> taskSituationScm(HashMap h) throws Exception;    
    
    
    
	/**
	 * 요구사항  목록 정보(Popup) 총 갯수
	 * @return int
	 * @exception Exception 
	*/
	public int getTskOdrListPopCnt(HashMap h) throws Exception;    
    
    
	/**
	 * 요구사항  목록 정보(Popup)
	 * @param HashMap - 조회할 정보가 담긴 HashMap
	 * @return 요구사항 목록
	 * @exception Exception
	 */
    public List getTskOdrListPop(HashMap h) throws Exception;     
    
    
}
