package egovframework.rte.dsdsm.usermg.service.impl;

import java.security.MessageDigest;
import java.sql.Clob;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.annotation.Resource;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Service;

import egovframework.rte.cmmn.util.EgovProperties;
import egovframework.rte.dsdsm.cmm.util.StrUtil;
import egovframework.rte.dsdsm.usermg.service.UserMgService;
import egovframework.rte.psl.dataaccess.util.EgovMap;

@Service("UserMgService")
public class UserMgServiceImpl implements UserMgService {
	
	@Resource(name="UserMgDAO")
	private UserMgDAO userMgDAO;
	
	private Logger log = LogManager.getLogger();

	@Override
	public Map<String, Object> modifyMyInfo(Map<String, Object> map) throws Exception{
		
		
		
		
		Map <String,Object> result = userMgDAO.modifyMyInfo(map);
		
		String databaseType = StrUtil.NVL(EgovProperties.getProperty("Globals.DatabaseType"),"postgres");
		
		// databaseType이 oracle인 경우 Clob 데이터 처리
		if("oracle".equals(databaseType)){
			userMgDAO.modifyMyInfoJobDelete(map);
			userMgDAO.modifyMyInfoJobInsert(map);
		}
		
		
		String isDutyCdChanged = (String)map.get("isDutyCdChanged");
		if( isDutyCdChanged!=null && "true".equals(isDutyCdChanged)){
			userMgDAO.deleteRoll(map);
			userMgDAO.insertRoll(map);
		}
		
		Integer cnt = (Integer)result.get("CNT");
		
		if(cnt!=null){
			result.put("SUCCESS", true);
			result.put("MESSAGE", "회원정보가 변경되었습니다.");
		}else{
			result.put("SUCCESS", false);
			result.put("MESSAGE", "회원정보변경에 실패하였습니다.");
		}
		
		return result;
	}
	
	@Override
	public Map<String, Object> modifyMyPassword(Map<String, String> map) throws Exception{
		
		Map <String,Object> result = userMgDAO.selectLoginPassword(map);
		Map one = (Map)result.get("ONE");
		
		String loginPassword = (String)one.get("loginPassword");
		String cyperCurrent = cypherString((String)map.get("current"));
		
		log.debug(loginPassword+","+cyperCurrent);
		
		if(loginPassword.equals(cyperCurrent)){
			map.put("loginPassword", cypherString(map.get("modified")));
			
			result = userMgDAO.updateLoginPassword(map);
			
			Integer cnt = (Integer)result.get("CNT");
			
			if(cnt!=null){
				result.put("SUCCESS", true);
				result.put("MESSAGE", "비밀번호가 변경되었습니다.");
			}else{
				result.put("SUCCESS", false);
				result.put("MESSAGE", "비밀번호 변경에 실패하였습니다.");
			}
			
		}else{
			result.put("SUCCESS", false);
			result.put("MESSAGE", "현재비밀번호가 일치하지 않습니다.");
		}
		
		return result;
	}
	
	@Override
	public Map<String, Object> approveUser(Map<String, Object> map) throws Exception {
		
		userMgDAO.deleteRoll(map);
		
		userMgDAO.insertRoll(map);
		
		Map <String,Object> result = userMgDAO.updateUserApproveYn(map);
		
		Integer cnt = (Integer)result.get("CNT");
		
		
		if(cnt!=null){
			result.put("SUCCESS", true);
			result.put("MESSAGE", "승인처리 성공");
		}else{
			result.put("SUCCESS", false);
			result.put("MESSAGE", "승인처리 실패");
		}
		
		result.remove("CNT");
		
		return result;
	}


	@Override
	public Map<String, Object> removeUser(Map<String, String> map) throws Exception {
		
		Map <String,Object> result = userMgDAO.updateUserDelYn(map);
		
		Integer cnt = (Integer)result.get("CNT");
		
		
		if(cnt!=null){
			result.put("SUCCESS", true);
			if("F".equals(map.get("delYn"))){
				result.put("MESSAGE", "강퇴처리 성공");
			}else{
				result.put("MESSAGE", "탈퇴처리 성공");
			}
		}else{
			result.put("SUCCESS", false);
			if("F".equals(map.get("delYn"))){
				result.put("MESSAGE", "강퇴처리 실패");
			}else{
				result.put("MESSAGE", "탈퇴처리 실패");
			}
		}
		
		result.remove("CNT");
		
		return result;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public List selectUsers(Map<String, Object> map) throws Exception{
		
		return userMgDAO.selectUsers(map);
		
	}
	
	@Override
	public Integer selectUsersCnt(Map<String, Object> map) throws Exception{
		
		return userMgDAO.selectUsersCnt(map);
		
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public Map<String,Object> selectUserInfo(Map<String,String> map) throws Exception{
		Map<String, Object> result = null;
		
		Map<String, Object> tempMap = userMgDAO.selectUserInfo(map);
		
		if(tempMap!=null){
			result = (Map<String, Object>)tempMap.get("ONE");
			
			///////////////////////////////////////////////////////////////////////////////////////////////////////////
			// databaseType이 oracle일 경우  postgres 에서 사용된 dataType array에 맞추어 String[]을 추가한다.
			String databaseType = StrUtil.NVL(EgovProperties.getProperty("Globals.DatabaseType"),"postgres");
			if(databaseType.equals("oracle")){
			String jobCdStr = (String)result.get("jobCdStr");
			String[] jobCds=jobCdStr.split(",");
			result.put("jobCd", jobCds);
			result.remove("jobCdStr");
			}
			///////////////////////////////////////////////////////////////////////////////////////////////////////////
		}
		return result;
	}
	
	@SuppressWarnings("unchecked")
	@Override
	public Map<String,Object> selectUserResetPassword(Map<String,String> map)  throws Exception{
		Map<String, Object> result = null;
		
		Map<String, Object> tempMap = userMgDAO.selectUserResetPassword(map);
		if(tempMap!=null){
			result = (Map<String, Object>)tempMap.get("ONE");
		}
		return result;
	}
	
	
	public String cypherString(String plain) throws Exception{
		String cypher = "";
		
		MessageDigest md = null;
		md = MessageDigest.getInstance("SHA-256");
		md.update(plain.getBytes());
		
		byte byteData[] = md.digest();
		StringBuffer sb = new StringBuffer();
		for(int i=0;i<byteData.length;i++){
			sb.append(Integer.toString((byteData[i]&0xff)+0x100,16).substring(1)); 
			//byte는 진수표현 함수없고 Integer의 유효숫자가 달라서 짤라줌
		}
		
		cypher = sb.toString();
		
		return cypher;
	}
	
}
