/**
 * Convert a single file-input element into a 'multiple' input list
 * Usage:
 *
 *   1. Create a file input element (no name)
 *      eg. <input type="file" id="first_file_element">
 *
 *   2. Create a DIV for the output to be written to
 *      eg. <div id="files_list"></div>
 *
 *   3. Instantiate a MultiSelector object, passing in the DIV and an (optional) maximum number of files
 *      eg. var multi_selector = new MultiSelector( document.getElementById( 'files_list' ), 3 );
 *
 *   4. Add the first element
 *      eg. multi_selector.addElement( document.getElementById( 'first_file_element' ) );
 */

function MultiSelector( list_target, max, maxSize){

	/*default 10mb. max 10mb */
	if(maxSize == null || maxSize == undefined || maxSize == "" || maxSize > 10) maxSize = 10;
	maxSize = maxSize * 1024 * 1024;
	// Where to write the list
	this.list_target = list_target;
	// How many elements?
	this.count = 0;
	// How many elements?
	this.id = 0;
	// Is there a maximum?
	if( max ){
		this.max = max;
	} else {
		this.max = -1;
	};
	
	
	/**
	 * Add a new file input element
	 */
	this.addElement = function( element ){
		

		
		// Make sure it's a file input element
		if( element.tagName == 'INPUT' && element.type == 'file' ){
			
			
			

			

			// Element name -- what number am I?
			element.name = 'file_' + this.id++;

			// Add reference to this object
			element.multi_selector = this;
			
			element.isCall = 0;
			
			// What to do when a file is selected
			element.onchange = function(){
				
				if(element.files[0].size==0){
					alert("파일 크기가 0입니다. 첨부파일을 확인해 주세요!");
					element.value="";
					return false;
				}
				
				
				if(element.isCall == 1) {
					console.log("already called.");
					return false;
				}
				
				element.isCall = 1;
				
				if(!chkFileSize(element, maxSize)) {				
					return false;
				}				

				// New file input
				var new_element = document.createElement( 'input' );
				new_element.type = 'file';

				// Add new element
				this.parentNode.insertBefore( new_element, this );

				// Apply 'update' to element
				this.multi_selector.addElement( new_element );

				// Update list
				this.multi_selector.addListRow( this );
				
				// Hide this: we can't use display:none because Safari doesn't like it
				this.style.position = 'absolute';
				this.style.left = '-1000px';
				this.style.top = '-1000px';
				this.style.display = 'none';
				this.style.visibility = 'hidden';
				this.style.width = '0';
				this.style.height = '0';
				this.style.overflow = 'hidden';

				new_element.onkeypress = function(){
					return false;
				};
				
				

			};
			// If we've reached maximum number, disable input element
			if( this.max != -1 && this.count >= this.max ){
				element.disabled = true;
			};

			// File element counter
			this.count++;
			// Most recent element
			this.current_element = element;
			
		} else {
			// This can only be applied to file input elements!
			alert( 'Error: not a file input element' );
		};

	};

	/**
	 * Add a new row to the list of files
	 */
	this.addListRow = function( element ){

		// Row div
		var new_row = document.createElement( 'div' );

		// Delete button
		var new_row_button = document.createElement( 'input' );
		new_row_button.type = 'button';
		new_row_button.value = 'Delete';

		// References
		new_row.element = element;

		// Delete function
		new_row_button.onclick= function(){

			// Remove element from form
			this.parentNode.element.parentNode.removeChild( this.parentNode.element );

			// Remove this row from the list
			this.parentNode.parentNode.removeChild( this.parentNode );

			// Decrement counter
			this.parentNode.element.multi_selector.count--;

			// Re-enable input element (if it's disabled)
			this.parentNode.element.multi_selector.current_element.disabled = false;

			//    which nixes your already queued uploads
			return false;
		};

		// Set row value
		new_row.innerHTML = "<div style='padding:2px;float:left;width:600px;' onMouseOver='this.style.background=\"#efefef\";' onMouseOut='this.style.background=\"#ffffff\";'  >"+element.value+"</div>";
		
		// Add button
		new_row.appendChild( new_row_button );

		// Add it to the list
		this.list_target.appendChild( new_row );
	};

};

/*fileSizeCheck*/
function chkFileSize(file, maxSize) {
	//var file = document.getElementsByName(fileNm)[0];
	if (file == null) {
		return false;
	} else {		
		
		var rtn = true;		
/*		console.log("#####FILE:::::"+file);
		console.log("#####MAX::::::"+maxSize);
		console.log("#####FILE LEN:"+file.files.length);		*/
		var agent = navigator.userAgent.toLowerCase();
//		console.log(agent);
		
		
		if(file.files.length == 1 && file.files[0].size > maxSize) {
			if (agent.indexOf("msie") != -1) {				
				file[0].select();
				document.selection.clear();
			} else {
				file.value = ""; 
			}
			alert("첨부파일은 "+Math.round((maxSize/1024)/1024)+"MB를 초과할 수 없습니다.");
			//console.log("too much FileSize::");
			rtn = false;
		} else if(file.files.length != 1) {
			if (agent.indexOf("msie") != -1) {				
				file[0].select();
				document.selection.clear();
			} else {
				file.value = ""; 
			}
			//console.log("File Length Error");
			rtn = false; 
		}
		
		
		file.isCall = 0;
		return rtn;
	} 	
}